@@ -53,36 +53,33 @@ gotoblas_t TABLE_NAME = {
 
   GEMM_DEFAULT_OFFSET_A, GEMM_DEFAULT_OFFSET_B, GEMM_DEFAULT_ALIGN,
 
+#ifdef BUILD_BFLOAT16
   0, 0, 0,
-  SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N,
-#ifdef SGEMM_DEFAULT_UNROLL_MN
- SGEMM_DEFAULT_UNROLL_MN,
+  SBGEMM_DEFAULT_UNROLL_M, SBGEMM_DEFAULT_UNROLL_N,
+#ifdef SBGEMM_DEFAULT_UNROLL_MN
+ SBGEMM_DEFAULT_UNROLL_MN,
 #else
- MAX(SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N),
+ MAX(SBGEMM_DEFAULT_UNROLL_M, SBGEMM_DEFAULT_UNROLL_N),
 #endif
 
-
-#ifdef HAVE_EXCLUSIVE_CACHE
-  1,
-#else
-  0,
-#endif
+  sbstobf16_k_CORE2, sbdtobf16_k_CORE2, sbf16tos_k_CORE2, dbf16tod_k_CORE2,
 
   samax_k_CORE2,  samin_k_CORE2,  smax_k_CORE2,  smin_k_CORE2,
   isamax_k_CORE2, isamin_k_CORE2, ismax_k_CORE2, ismin_k_CORE2,
-  snrm2_k_CORE2,  sasum_k_CORE2,  scopy_k_CORE2, sdot_k_CORE2,
+  snrm2_k_CORE2,  sasum_k_CORE2, ssum_k_CORE2, scopy_k_CORE2, sbdot_k_CORE2,
   dsdot_k_CORE2,
   srot_k_CORE2,   saxpy_k_CORE2,  sscal_k_CORE2, sswap_k_CORE2,
-  sgemv_n_CORE2,  sgemv_t_CORE2, sger_k_CORE2,
+  sbgemv_n_CORE2, sbgemv_t_CORE2, sger_k_CORE2,
   ssymv_L_CORE2, ssymv_U_CORE2,
 
-  sgemm_kernel_CORE2, sgemm_beta_CORE2,
-#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
-  sgemm_incopy_CORE2, sgemm_itcopy_CORE2,
+  sbgemm_kernel_CORE2, sbgemm_beta_CORE2,
+#if SBGEMM_DEFAULT_UNROLL_M != SBGEMM_DEFAULT_UNROLL_N
+  sbgemm_incopy_CORE2, sbgemm_itcopy_CORE2,
 #else
-  sgemm_oncopy_CORE2, sgemm_otcopy_CORE2,
+  sbgemm_oncopy_CORE2, sbgemm_otcopy_CORE2,
 #endif
-  sgemm_oncopy_CORE2, sgemm_otcopy_CORE2,
+  sbgemm_oncopy_CORE2, sbgemm_otcopy_CORE2,
+
   strsm_kernel_LN_CORE2, strsm_kernel_LT_CORE2, strsm_kernel_RN_CORE2, strsm_kernel_RT_CORE2,
 #if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
   strsm_iunucopy_CORE2, strsm_iunncopy_CORE2, strsm_iutucopy_CORE2, strsm_iutncopy_CORE2,
@@ -115,7 +112,104 @@ gotoblas_t TABLE_NAME = {
 #else
   NULL,NULL,
 #endif
+#endif
 
+#if ( BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1) || (BUILD_COMPLEX16==1)
+  0, 0, 0,
+  SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N,
+#ifdef SGEMM_DEFAULT_UNROLL_MN
+ SGEMM_DEFAULT_UNROLL_MN,
+#else
+ MAX(SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N),
+#endif
+#endif
+
+#ifdef HAVE_EXCLUSIVE_CACHE
+  1,
+#else
+  0,
+#endif
+
+#if (BUILD_SINGLE==1 ) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  samax_k_CORE2,  samin_k_CORE2,  smax_k_CORE2,  smin_k_CORE2,
+  isamax_k_CORE2, isamin_k_CORE2, ismax_k_CORE2, ismin_k_CORE2,
+  snrm2_k_CORE2,  sasum_k_CORE2,
+#endif 
+#if BUILD_SINGLE == 1  
+  ssum_k_CORE2,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  scopy_k_CORE2, sdot_k_CORE2,
+//  dsdot_k_CORE2,
+  srot_k_CORE2,   saxpy_k_CORE2,  
+#endif
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1) || (BUILD_COMPLEX16==1)
+  sscal_k_CORE2,
+#endif 
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  sswap_k_CORE2,
+  sgemv_n_CORE2,  sgemv_t_CORE2,
+#endif
+#if BUILD_SINGLE == 1  
+  sger_k_CORE2,
+  ssymv_L_CORE2, ssymv_U_CORE2,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+#ifdef ARCH_X86_64
+  sgemm_direct_CORE2,
+  sgemm_direct_performant_CORE2,	
+#endif
+
+  sgemm_kernel_CORE2, sgemm_beta_CORE2,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  sgemm_incopy_CORE2, sgemm_itcopy_CORE2,
+#else
+  sgemm_oncopy_CORE2, sgemm_otcopy_CORE2,
+#endif
+  sgemm_oncopy_CORE2, sgemm_otcopy_CORE2,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) 
+  strsm_kernel_LN_CORE2, strsm_kernel_LT_CORE2, strsm_kernel_RN_CORE2, strsm_kernel_RT_CORE2,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  strsm_iunucopy_CORE2, strsm_iunncopy_CORE2, strsm_iutucopy_CORE2, strsm_iutncopy_CORE2,
+  strsm_ilnucopy_CORE2, strsm_ilnncopy_CORE2, strsm_iltucopy_CORE2, strsm_iltncopy_CORE2,
+#else
+  strsm_ounucopy_CORE2, strsm_ounncopy_CORE2, strsm_outucopy_CORE2, strsm_outncopy_CORE2,
+  strsm_olnucopy_CORE2, strsm_olnncopy_CORE2, strsm_oltucopy_CORE2, strsm_oltncopy_CORE2,
+#endif
+  strsm_ounucopy_CORE2, strsm_ounncopy_CORE2, strsm_outucopy_CORE2, strsm_outncopy_CORE2,
+  strsm_olnucopy_CORE2, strsm_olnncopy_CORE2, strsm_oltucopy_CORE2, strsm_oltncopy_CORE2,
+#endif
+#if BUILD_SINGLE == 1
+  strmm_kernel_RN_CORE2, strmm_kernel_RT_CORE2, strmm_kernel_LN_CORE2, strmm_kernel_LT_CORE2,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  strmm_iunucopy_CORE2, strmm_iunncopy_CORE2, strmm_iutucopy_CORE2, strmm_iutncopy_CORE2,
+  strmm_ilnucopy_CORE2, strmm_ilnncopy_CORE2, strmm_iltucopy_CORE2, strmm_iltncopy_CORE2,
+#else
+  strmm_ounucopy_CORE2, strmm_ounncopy_CORE2, strmm_outucopy_CORE2, strmm_outncopy_CORE2,
+  strmm_olnucopy_CORE2, strmm_olnncopy_CORE2, strmm_oltucopy_CORE2, strmm_oltncopy_CORE2,
+#endif
+  strmm_ounucopy_CORE2, strmm_ounncopy_CORE2, strmm_outucopy_CORE2, strmm_outncopy_CORE2,
+  strmm_olnucopy_CORE2, strmm_olnncopy_CORE2, strmm_oltucopy_CORE2, strmm_oltncopy_CORE2,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  ssymm_iutcopy_CORE2, ssymm_iltcopy_CORE2,
+#else
+  ssymm_outcopy_CORE2, ssymm_oltcopy_CORE2,
+#endif
+  ssymm_outcopy_CORE2, ssymm_oltcopy_CORE2,
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_DOUBLE==1)
+#ifndef NO_LAPACK
+  sneg_tcopy_CORE2, slaswp_ncopy_CORE2,
+#else
+  NULL,NULL,
+#endif
+#endif
+
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   0, 0, 0,
   DGEMM_DEFAULT_UNROLL_M, DGEMM_DEFAULT_UNROLL_N,
 #ifdef DGEMM_DEFAULT_UNROLL_MN
@@ -123,14 +217,36 @@ gotoblas_t TABLE_NAME = {
 #else
  MAX(DGEMM_DEFAULT_UNROLL_M, DGEMM_DEFAULT_UNROLL_N),
 #endif
+#endif
 
+
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   damax_k_CORE2,  damin_k_CORE2,  dmax_k_CORE2,  dmin_k_CORE2,
   idamax_k_CORE2, idamin_k_CORE2, idmax_k_CORE2, idmin_k_CORE2,
-  dnrm2_k_CORE2,  dasum_k_CORE2,  dcopy_k_CORE2, ddot_k_CORE2,
-  drot_k_CORE2,   daxpy_k_CORE2,  dscal_k_CORE2, dswap_k_CORE2,
-  dgemv_n_CORE2,  dgemv_t_CORE2,  dger_k_CORE2,
+  dnrm2_k_CORE2, dasum_k_CORE2,
+#endif  
+#if  (BUILD_DOUBLE==1)  
+  dsum_k_CORE2,
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
+  dcopy_k_CORE2, ddot_k_CORE2,
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_DOUBLE==1)  
+  dsdot_k_CORE2,
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
+  drot_k_CORE2,
+  daxpy_k_CORE2,
+  dscal_k_CORE2, 
+  dswap_k_CORE2,
+  dgemv_n_CORE2,  dgemv_t_CORE2,
+#endif
+#if  (BUILD_DOUBLE==1)  
+  dger_k_CORE2,
   dsymv_L_CORE2,  dsymv_U_CORE2,
+#endif
 
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   dgemm_kernel_CORE2, dgemm_beta_CORE2,
 #if DGEMM_DEFAULT_UNROLL_M != DGEMM_DEFAULT_UNROLL_N
   dgemm_incopy_CORE2, dgemm_itcopy_CORE2,
@@ -138,6 +254,9 @@ gotoblas_t TABLE_NAME = {
   dgemm_oncopy_CORE2, dgemm_otcopy_CORE2,
 #endif
   dgemm_oncopy_CORE2, dgemm_otcopy_CORE2,
+#endif
+
+#if  (BUILD_DOUBLE==1)  
   dtrsm_kernel_LN_CORE2, dtrsm_kernel_LT_CORE2, dtrsm_kernel_RN_CORE2, dtrsm_kernel_RT_CORE2,
 #if DGEMM_DEFAULT_UNROLL_M != DGEMM_DEFAULT_UNROLL_N
   dtrsm_iunucopy_CORE2, dtrsm_iunncopy_CORE2, dtrsm_iutucopy_CORE2, dtrsm_iutncopy_CORE2,
@@ -171,6 +290,8 @@ gotoblas_t TABLE_NAME = {
   NULL, NULL,
 #endif
 
+#endif
+
 #ifdef EXPRECISION
 
   0, 0, 0,
@@ -178,7 +299,7 @@ gotoblas_t TABLE_NAME = {
 
   qamax_k_CORE2,  qamin_k_CORE2,  qmax_k_CORE2,  qmin_k_CORE2,
   iqamax_k_CORE2, iqamin_k_CORE2, iqmax_k_CORE2, iqmin_k_CORE2,
-  qnrm2_k_CORE2,  qasum_k_CORE2,  qcopy_k_CORE2, qdot_k_CORE2,
+  qnrm2_k_CORE2,  qasum_k_CORE2,  qsum_k_CORE2, qcopy_k_CORE2, qdot_k_CORE2,
   qrot_k_CORE2,   qaxpy_k_CORE2,  qscal_k_CORE2, qswap_k_CORE2,
   qgemv_n_CORE2,  qgemv_t_CORE2,  qger_k_CORE2,
   qsymv_L_CORE2,  qsymv_U_CORE2,
@@ -225,6 +346,7 @@ gotoblas_t TABLE_NAME = {
 
 #endif
 
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
   0, 0, 0,
   CGEMM_DEFAULT_UNROLL_M, CGEMM_DEFAULT_UNROLL_N,
 #ifdef CGEMM_DEFAULT_UNROLL_MN
@@ -232,21 +354,34 @@ gotoblas_t TABLE_NAME = {
 #else
  MAX(CGEMM_DEFAULT_UNROLL_M, CGEMM_DEFAULT_UNROLL_N),
 #endif
-
   camax_k_CORE2, camin_k_CORE2, icamax_k_CORE2, icamin_k_CORE2,
-  cnrm2_k_CORE2, casum_k_CORE2, ccopy_k_CORE2,
-  cdotu_k_CORE2, cdotc_k_CORE2, csrot_k_CORE2,
-  caxpy_k_CORE2, caxpyc_k_CORE2, cscal_k_CORE2, cswap_k_CORE2,
+#endif
+#if (BUILD_COMPLEX)
+  cnrm2_k_CORE2, casum_k_CORE2, csum_k_CORE2,
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
+  ccopy_k_CORE2,  cdotu_k_CORE2, cdotc_k_CORE2,
+#endif
+#if (BUILD_COMPLEX)
+ csrot_k_CORE2,
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
+  caxpy_k_CORE2,
+  caxpyc_k_CORE2, 
+  cscal_k_CORE2, 
+  cswap_k_CORE2,
 
   cgemv_n_CORE2, cgemv_t_CORE2, cgemv_r_CORE2, cgemv_c_CORE2,
   cgemv_o_CORE2, cgemv_u_CORE2, cgemv_s_CORE2, cgemv_d_CORE2,
+#endif
+#if (BUILD_COMPLEX)
   cgeru_k_CORE2, cgerc_k_CORE2, cgerv_k_CORE2, cgerd_k_CORE2,
   csymv_L_CORE2, csymv_U_CORE2,
   chemv_L_CORE2, chemv_U_CORE2, chemv_M_CORE2, chemv_V_CORE2,
-
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
   cgemm_kernel_n_CORE2, cgemm_kernel_l_CORE2, cgemm_kernel_r_CORE2, cgemm_kernel_b_CORE2,
   cgemm_beta_CORE2,
-
 #if CGEMM_DEFAULT_UNROLL_M != CGEMM_DEFAULT_UNROLL_N
   cgemm_incopy_CORE2, cgemm_itcopy_CORE2,
 #else
@@ -266,6 +401,8 @@ gotoblas_t TABLE_NAME = {
 #endif
   ctrsm_ounucopy_CORE2,  ctrsm_ounncopy_CORE2,  ctrsm_outucopy_CORE2,  ctrsm_outncopy_CORE2,
   ctrsm_olnucopy_CORE2,  ctrsm_olnncopy_CORE2,  ctrsm_oltucopy_CORE2,  ctrsm_oltncopy_CORE2,
+#endif
+#if (BUILD_COMPLEX)
 
   ctrmm_kernel_RN_CORE2,  ctrmm_kernel_RT_CORE2,  ctrmm_kernel_RR_CORE2,  ctrmm_kernel_RC_CORE2,
   ctrmm_kernel_LN_CORE2,  ctrmm_kernel_LT_CORE2,  ctrmm_kernel_LR_CORE2,  ctrmm_kernel_LC_CORE2,
@@ -294,6 +431,8 @@ gotoblas_t TABLE_NAME = {
   chemm_outcopy_CORE2,  chemm_oltcopy_CORE2,
 
   0, 0, 0,
+
+#if (USE_GEMM3M)
 #ifdef CGEMM3M_DEFAULT_UNROLL_M
   CGEMM3M_DEFAULT_UNROLL_M, CGEMM3M_DEFAULT_UNROLL_N, MAX(CGEMM3M_DEFAULT_UNROLL_M, CGEMM3M_DEFAULT_UNROLL_N),
 #else
@@ -324,13 +463,47 @@ gotoblas_t TABLE_NAME = {
   chemm3m_oucopyb_CORE2,  chemm3m_olcopyb_CORE2,
   chemm3m_oucopyr_CORE2,  chemm3m_olcopyr_CORE2,
   chemm3m_oucopyi_CORE2,  chemm3m_olcopyi_CORE2,
+#else
+  0, 0, 0,
 
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
+#endif
+
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
 #ifndef NO_LAPACK
-  cneg_tcopy_CORE2, claswp_ncopy_CORE2,
+  cneg_tcopy_CORE2,
+  
+   claswp_ncopy_CORE2,
 #else
   NULL, NULL,
 #endif
 
+#endif
+
+#if BUILD_COMPLEX16 == 1
   0, 0, 0,
   ZGEMM_DEFAULT_UNROLL_M, ZGEMM_DEFAULT_UNROLL_N,
 #ifdef ZGEMM_DEFAULT_UNROLL_MN
@@ -340,7 +513,7 @@ gotoblas_t TABLE_NAME = {
 #endif
 
   zamax_k_CORE2, zamin_k_CORE2, izamax_k_CORE2, izamin_k_CORE2,
-  znrm2_k_CORE2, zasum_k_CORE2, zcopy_k_CORE2,
+  znrm2_k_CORE2, zasum_k_CORE2, zsum_k_CORE2, zcopy_k_CORE2,
   zdotu_k_CORE2, zdotc_k_CORE2, zdrot_k_CORE2,
   zaxpy_k_CORE2, zaxpyc_k_CORE2, zscal_k_CORE2, zswap_k_CORE2,
 
@@ -400,6 +573,7 @@ gotoblas_t TABLE_NAME = {
   zhemm_outcopy_CORE2,  zhemm_oltcopy_CORE2,
 
   0, 0, 0,
+#if (USE_GEMM3M)
 #ifdef ZGEMM3M_DEFAULT_UNROLL_M
   ZGEMM3M_DEFAULT_UNROLL_M, ZGEMM3M_DEFAULT_UNROLL_N, MAX(ZGEMM3M_DEFAULT_UNROLL_M, ZGEMM3M_DEFAULT_UNROLL_N),
 #else
@@ -430,6 +604,33 @@ gotoblas_t TABLE_NAME = {
   zhemm3m_oucopyb_CORE2,  zhemm3m_olcopyb_CORE2,
   zhemm3m_oucopyr_CORE2,  zhemm3m_olcopyr_CORE2,
   zhemm3m_oucopyi_CORE2,  zhemm3m_olcopyi_CORE2,
+#else
+  0, 0, 0,
+
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
 
 #ifndef NO_LAPACK
   zneg_tcopy_CORE2, zlaswp_ncopy_CORE2,
@@ -437,13 +638,15 @@ gotoblas_t TABLE_NAME = {
   NULL, NULL,
 #endif
 
+#endif
+
 #ifdef EXPRECISION
 
   0, 0, 0,
   XGEMM_DEFAULT_UNROLL_M, XGEMM_DEFAULT_UNROLL_N, MAX(XGEMM_DEFAULT_UNROLL_M, XGEMM_DEFAULT_UNROLL_N),
 
   xamax_k_CORE2, xamin_k_CORE2, ixamax_k_CORE2, ixamin_k_CORE2,
-  xnrm2_k_CORE2, xasum_k_CORE2, xcopy_k_CORE2,
+  xnrm2_k_CORE2, xasum_k_CORE2, xsum_k_CORE2, xcopy_k_CORE2,
   xdotu_k_CORE2, xdotc_k_CORE2, xqrot_k_CORE2,
   xaxpy_k_CORE2, xaxpyc_k_CORE2, xscal_k_CORE2, xswap_k_CORE2,
 
@@ -503,6 +706,7 @@ gotoblas_t TABLE_NAME = {
   xhemm_outcopy_CORE2,  xhemm_oltcopy_CORE2,
 
   0, 0, 0,
+#if (USE_GEMM3M)
   QGEMM_DEFAULT_UNROLL_M, QGEMM_DEFAULT_UNROLL_N, MAX(QGEMM_DEFAULT_UNROLL_M, QGEMM_DEFAULT_UNROLL_N),
 
   xgemm3m_kernel_CORE2,
@@ -528,6 +732,33 @@ gotoblas_t TABLE_NAME = {
   xhemm3m_oucopyb_CORE2,  xhemm3m_olcopyb_CORE2,
   xhemm3m_oucopyr_CORE2,  xhemm3m_olcopyr_CORE2,
   xhemm3m_oucopyi_CORE2,  xhemm3m_olcopyi_CORE2,
+#else
+  0, 0, 0,
+
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
 
 #ifndef NO_LAPACK
   xneg_tcopy_CORE2, xlaswp_ncopy_CORE2,
@@ -540,27 +771,298 @@ gotoblas_t TABLE_NAME = {
   init_parameter,
 
   SNUMOPT, DNUMOPT, QNUMOPT,
+#if BUILD_SINGLE == 1
+  saxpby_k_CORE2,
+#endif
+#if BUILD_DOUBLE  == 1
+  daxpby_k_CORE2,
+#endif
+#if BUILD_COMPLEX == 1
+  caxpby_k_CORE2,
+#endif
+#if BUILD_COMPLEX16== 1
+  zaxpby_k_CORE2,
+#endif
 
-  saxpby_k_CORE2, daxpby_k_CORE2, caxpby_k_CORE2, zaxpby_k_CORE2,
-
+#if BUILD_SINGLE == 1
   somatcopy_k_cn_CORE2, somatcopy_k_ct_CORE2, somatcopy_k_rn_CORE2, somatcopy_k_rt_CORE2,
+#endif
+#if BUILD_DOUBLE== 1
   domatcopy_k_cn_CORE2, domatcopy_k_ct_CORE2, domatcopy_k_rn_CORE2, domatcopy_k_rt_CORE2,
+#endif
+#if BUILD_COMPLEX == 1
   comatcopy_k_cn_CORE2, comatcopy_k_ct_CORE2, comatcopy_k_rn_CORE2, comatcopy_k_rt_CORE2,
   comatcopy_k_cnc_CORE2, comatcopy_k_ctc_CORE2, comatcopy_k_rnc_CORE2, comatcopy_k_rtc_CORE2,
+#endif
+#if BUILD_COMPLEX16 == 1
   zomatcopy_k_cn_CORE2, zomatcopy_k_ct_CORE2, zomatcopy_k_rn_CORE2, zomatcopy_k_rt_CORE2,
   zomatcopy_k_cnc_CORE2, zomatcopy_k_ctc_CORE2, zomatcopy_k_rnc_CORE2, zomatcopy_k_rtc_CORE2,
+#endif
 
+#if BUILD_SINGLE == 1
   simatcopy_k_cn_CORE2, simatcopy_k_ct_CORE2, simatcopy_k_rn_CORE2, simatcopy_k_rt_CORE2,
+#endif
+#if BUILD_DOUBLE== 1
   dimatcopy_k_cn_CORE2, dimatcopy_k_ct_CORE2, dimatcopy_k_rn_CORE2, dimatcopy_k_rt_CORE2,
+#endif
+#if BUILD_COMPLEX== 1
   cimatcopy_k_cn_CORE2, cimatcopy_k_ct_CORE2, cimatcopy_k_rn_CORE2, cimatcopy_k_rt_CORE2,
   cimatcopy_k_cnc_CORE2, cimatcopy_k_ctc_CORE2, cimatcopy_k_rnc_CORE2, cimatcopy_k_rtc_CORE2,
+#endif
+#if BUILD_COMPLEX16==1
   zimatcopy_k_cn_CORE2, zimatcopy_k_ct_CORE2, zimatcopy_k_rn_CORE2, zimatcopy_k_rt_CORE2,
   zimatcopy_k_cnc_CORE2, zimatcopy_k_ctc_CORE2, zimatcopy_k_rnc_CORE2, zimatcopy_k_rtc_CORE2,
+#endif
 
-  sgeadd_k_CORE2, dgeadd_k_CORE2, cgeadd_k_CORE2, zgeadd_k_CORE2
-
+#if BUILD_SINGLE == 1
+  sgeadd_k_CORE2,
+#endif
+#if BUILD_DOUBLE==1
+  dgeadd_k_CORE2,
+#endif
+#if BUILD_COMPLEX==1
+  cgeadd_k_CORE2,
+#endif
+#if BUILD_COMPLEX16==1
+  zgeadd_k_CORE2
+#endif
 };
 
+#if (ARCH_ARM64)
+static void init_parameter(void) {
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+#if (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE == 1
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+#endif
+#if BUILD_DOUBLE== 1
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX== 1
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+#endif
+
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+#endif
+#if BUILD_DOUBLE==1 
+  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+  TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
+  TABLE_NAME.xgemm_q = XGEMM_DEFAULT_Q;
+  TABLE_NAME.qgemm_r = QGEMM_DEFAULT_R;
+  TABLE_NAME.xgemm_r = XGEMM_DEFAULT_R;
+#endif
+
+#if (USE_GEMM3M)
+#ifdef CGEMM3M_DEFAULT_P
+  TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_P
+  TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_Q
+  TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.cgemm3m_q = TABLE_NAME.sgemm_q;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_Q
+  TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.zgemm3m_q = TABLE_NAME.dgemm_q;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_R
+  TABLE_NAME.cgemm3m_r = CGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.cgemm3m_r = TABLE_NAME.sgemm_r;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_R
+  TABLE_NAME.zgemm3m_r = ZGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.zgemm3m_r = TABLE_NAME.dgemm_r;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
+  TABLE_NAME.xgemm3m_q = TABLE_NAME.qgemm_q;
+  TABLE_NAME.xgemm3m_r = TABLE_NAME.qgemm_r;
+#endif
+#endif
+
+}
+#else // (ARCH_ARM64)
+#if defined(ARCH_MIPS64)
+static void init_parameter(void) {
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+  TABLE_NAME.dgemm_r = 640;
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+  TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
+  TABLE_NAME.xgemm_q = XGEMM_DEFAULT_Q;
+  TABLE_NAME.qgemm_r = QGEMM_DEFAULT_R;
+  TABLE_NAME.xgemm_r = XGEMM_DEFAULT_R;
+#endif
+
+#if defined(USE_GEMM3M)
+#ifdef CGEMM3M_DEFAULT_P
+  TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_P
+  TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_Q
+  TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.cgemm3m_q = TABLE_NAME.sgemm_q;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_Q
+  TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.zgemm3m_q = TABLE_NAME.dgemm_q;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_R
+  TABLE_NAME.cgemm3m_r = CGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.cgemm3m_r = TABLE_NAME.sgemm_r;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_R
+  TABLE_NAME.zgemm3m_r = ZGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.zgemm3m_r = TABLE_NAME.dgemm_r;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
+  TABLE_NAME.xgemm3m_q = TABLE_NAME.qgemm_q;
+  TABLE_NAME.xgemm3m_r = TABLE_NAME.qgemm_r;
+#endif
+#endif
+}
+#else // (ARCH_MIPS64)
+#if (ARCH_POWER)
+static void init_parameter(void) {
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+}
+#else //POWER
+
+#if (ARCH_ZARCH)
+static void init_parameter(void) {
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+	TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+	TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+	TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+	TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+	TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+	TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+	TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+	TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+	TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+	TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+	TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+	TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+}
+#else //ZARCH
+
 #ifdef ARCH_X86
 static int get_l2_size_old(void){
   int i, eax, ebx, ecx, edx, cpuid_level;
@@ -647,7 +1149,9 @@ static int get_l2_size_old(void){
       return 6144;
     }
   }
-  return 0;
+//  return 0;
+fprintf (stderr,"OpenBLAS WARNING - could not determine the L2 cache size on this system, assuming 256k\n");
+return 256;
 }
 #endif
 
@@ -660,6 +1164,10 @@ static __inline__ int get_l2_size(void){
   l2 = BITMASK(ecx, 16, 0xffff);
 
 #ifndef ARCH_X86
+  if (l2 <= 0) {
+     fprintf (stderr,"OpenBLAS WARNING - could not determine the L2 cache size on this system, assuming 256k\n");
+     return 256;
+  }
   return l2;
 
 #else
@@ -684,22 +1192,42 @@ static void init_parameter(void) {
 
   int l2 = get_l2_size();
 
+  (void) l2; /* dirty trick to suppress unused variable warning for targets */
+             /* where the GEMM unrolling parameters do not depend on l2 */
+  
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)
   TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX == 1
   TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+#endif
 
+#if BUILD_COMPLEX == 1
 #ifdef CGEMM3M_DEFAULT_Q
   TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
 #else
   TABLE_NAME.cgemm3m_q = SGEMM_DEFAULT_Q;
 #endif
+#endif
 
+#if BUILD_COMPLEX16 == 1
 #ifdef ZGEMM3M_DEFAULT_Q
   TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
 #else
   TABLE_NAME.zgemm3m_q = DGEMM_DEFAULT_Q;
 #endif
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
@@ -713,10 +1241,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Katmai, Coppermine, Banias, Athlon\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  64 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  32 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1 
   TABLE_NAME.cgemm_p =  32 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  16 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  16 * (l2 >> 7);
   TABLE_NAME.xgemm_p =   8 * (l2 >> 7);
@@ -729,10 +1265,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Northwood\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  96 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  48 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1 
   TABLE_NAME.cgemm_p =  48 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  24 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  24 * (l2 >> 7);
   TABLE_NAME.xgemm_p =  12 * (l2 >> 7);
@@ -745,10 +1289,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Atom\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = 256;
+#endif
+#if BUILD_DOUBLE ==1
   TABLE_NAME.dgemm_p = 128;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p = 128;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  64;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  64;
   TABLE_NAME.xgemm_p =  32;
@@ -761,10 +1313,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Prescott\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  56 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE ==1 
   TABLE_NAME.dgemm_p =  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16 == 1
   TABLE_NAME.zgemm_p =  14 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  14 * (l2 >> 7);
   TABLE_NAME.xgemm_p =   7 * (l2 >> 7);
@@ -777,10 +1337,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Core2\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  92 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE==1
   TABLE_NAME.dgemm_p =  46 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  46 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  23 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  92 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  46 * (l2 >> 9) + 4;
@@ -793,10 +1361,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Penryn\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  21 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  21 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  42 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  21 * (l2 >> 9) + 4;
@@ -809,10 +1385,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Dunnington\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE ==1
   TABLE_NAME.dgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  21 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  21 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  42 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  21 * (l2 >> 9) + 4;
@@ -826,10 +1410,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Nehalem\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -842,10 +1434,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Sandybridge\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -858,10 +1458,42 @@ static void init_parameter(void) {
   fprintf(stderr, "Haswell\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+#if defined(SKYLAKEX) || defined(COOPERLAKE)
+
+#ifdef DEBUG
+  fprintf(stderr, "SkylakeX\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -875,10 +1507,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Opteron\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = 224 +  56 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = 112 +  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = 112 +  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p =  56 +  14 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  56 +  14 * (l2 >> 7);
   TABLE_NAME.xgemm_p =  28 +   7 * (l2 >> 7);
@@ -891,10 +1531,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Barcelona\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -907,10 +1555,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Bobcate\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -923,26 +1579,67 @@ static void init_parameter(void) {
   fprintf(stderr, "Bulldozer\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
 #endif
 #endif
 
+#ifdef EXCAVATOR
+
+#ifdef DEBUG
+  fprintf(stderr, "Excavator\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+
 #ifdef PILEDRIVER
 
 #ifdef DEBUG
   fprintf(stderr, "Piledriver\n");
 #endif
 
+#if (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -955,10 +1652,42 @@ static void init_parameter(void) {
   fprintf(stderr, "Steamroller\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+#ifdef ZEN
+
+#ifdef DEBUG
+  fprintf(stderr, "Zen\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -972,11 +1701,18 @@ static void init_parameter(void) {
   fprintf(stderr, "NANO\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if  (BUILD_DOUBLE==1)
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if (BUILD_COMPLEX==1)
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if (BUILD_COMPLEX16==1)
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
-
+#endif
 
 
 #ifdef EXPRECISION
@@ -986,61 +1722,79 @@ static void init_parameter(void) {
 
 #endif
 
-
+#if BUILD_COMPLEX==1
 #ifdef CGEMM3M_DEFAULT_P
   TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
 #else
   TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
 #endif
+#endif
 
+#if BUILD_COMPLEX16==1
 #ifdef ZGEMM3M_DEFAULT_P
   TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
 #else
   TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
 #endif
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
 #endif
 
 
-
-  TABLE_NAME.sgemm_p = (TABLE_NAME.sgemm_p + SGEMM_DEFAULT_UNROLL_M - 1) & ~(SGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.dgemm_p = (TABLE_NAME.dgemm_p + DGEMM_DEFAULT_UNROLL_M - 1) & ~(DGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.cgemm_p = (TABLE_NAME.cgemm_p + CGEMM_DEFAULT_UNROLL_M - 1) & ~(CGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.zgemm_p = (TABLE_NAME.zgemm_p + ZGEMM_DEFAULT_UNROLL_M - 1) & ~(ZGEMM_DEFAULT_UNROLL_M - 1);
-
-#ifdef CGEMM3M_DEFAULT_UNROLL_M
-  TABLE_NAME.cgemm3m_p = (TABLE_NAME.cgemm3m_p + CGEMM3M_DEFAULT_UNROLL_M - 1) & ~(CGEMM3M_DEFAULT_UNROLL_M - 1);
-#else
-  TABLE_NAME.cgemm3m_p = (TABLE_NAME.cgemm3m_p + SGEMM_DEFAULT_UNROLL_M - 1) & ~(SGEMM_DEFAULT_UNROLL_M - 1);
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_p = ((TABLE_NAME.sgemm_p + SGEMM_DEFAULT_UNROLL_M - 1)/SGEMM_DEFAULT_UNROLL_M) * SGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_DOUBLE== 1
+  TABLE_NAME.dgemm_p = ((TABLE_NAME.dgemm_p + DGEMM_DEFAULT_UNROLL_M - 1)/DGEMM_DEFAULT_UNROLL_M) * DGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_p = ((TABLE_NAME.cgemm_p + CGEMM_DEFAULT_UNROLL_M - 1)/CGEMM_DEFAULT_UNROLL_M) * CGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_p = ((TABLE_NAME.zgemm_p + ZGEMM_DEFAULT_UNROLL_M - 1)/ZGEMM_DEFAULT_UNROLL_M) * ZGEMM_DEFAULT_UNROLL_M;
 #endif
 
-#ifdef ZGEMM3M_DEFAULT_UNROLL_M
-  TABLE_NAME.zgemm3m_p = (TABLE_NAME.zgemm3m_p + ZGEMM3M_DEFAULT_UNROLL_M - 1) & ~(ZGEMM3M_DEFAULT_UNROLL_M - 1);
+#if BUILD_COMPLEX==1
+#ifdef CGEMM3M_DEFAULT_UNROLL_M
+  TABLE_NAME.cgemm3m_p = ((TABLE_NAME.cgemm3m_p + CGEMM3M_DEFAULT_UNROLL_M - 1)/CGEMM3M_DEFAULT_UNROLL_M) * CGEMM3M_DEFAULT_UNROLL_M;
 #else
-  TABLE_NAME.zgemm3m_p = (TABLE_NAME.zgemm3m_p + DGEMM_DEFAULT_UNROLL_M - 1) & ~(DGEMM_DEFAULT_UNROLL_M - 1);
+  TABLE_NAME.cgemm3m_p = ((TABLE_NAME.cgemm3m_p + SGEMM_DEFAULT_UNROLL_M - 1)/SGEMM_DEFAULT_UNROLL_M) * SGEMM_DEFAULT_UNROLL_M;
+#endif
+#endif
+
+#if BUILD_COMPLEX16==1
+#ifdef ZGEMM3M_DEFAULT_UNROLL_M
+  TABLE_NAME.zgemm3m_p = ((TABLE_NAME.zgemm3m_p + ZGEMM3M_DEFAULT_UNROLL_M - 1)/ZGEMM3M_DEFAULT_UNROLL_M) * ZGEMM3M_DEFAULT_UNROLL_M;
+#else
+  TABLE_NAME.zgemm3m_p = ((TABLE_NAME.zgemm3m_p + DGEMM_DEFAULT_UNROLL_M - 1)/DGEMM_DEFAULT_UNROLL_M) * DGEMM_DEFAULT_UNROLL_M;
+#endif
 #endif
 
 #ifdef QUAD_PRECISION
-  TABLE_NAME.qgemm_p = (TABLE_NAME.qgemm_p + QGEMM_DEFAULT_UNROLL_M - 1) & ~(QGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.xgemm_p = (TABLE_NAME.xgemm_p + XGEMM_DEFAULT_UNROLL_M - 1) & ~(XGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.xgemm3m_p = (TABLE_NAME.xgemm3m_p + QGEMM_DEFAULT_UNROLL_M - 1) & ~(QGEMM_DEFAULT_UNROLL_M - 1);
+  TABLE_NAME.qgemm_p = ((TABLE_NAME.qgemm_p + QGEMM_DEFAULT_UNROLL_M - 1)/QGEMM_DEFAULT_UNROLL_M) * QGEMM_DEFAULT_UNROLL_M;
+  TABLE_NAME.xgemm_p = ((TABLE_NAME.xgemm_p + XGEMM_DEFAULT_UNROLL_M - 1)/XGEMM_DEFAULT_UNROLL_M) * XGEMM_DEFAULT_UNROLL_M;
+  TABLE_NAME.xgemm3m_p = ((TABLE_NAME.xgemm3m_p + QGEMM_DEFAULT_UNROLL_M - 1)/QGEMM_DEFAULT_UNROLL_M) * QGEMM_DEFAULT_UNROLL_M;
 #endif
 
 #ifdef DEBUG
   fprintf(stderr, "L2 = %8d DGEMM_P  .. %d\n", l2, TABLE_NAME.dgemm_p);
 #endif
 
+#if BUILD_SINGLE==1
   TABLE_NAME.sgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.sgemm_p * TABLE_NAME.sgemm_q *  4 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.sgemm_q *  4) - 15) & ~15);
+#endif
 
+#if BUILD_DOUBLE==1
   TABLE_NAME.dgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.dgemm_p * TABLE_NAME.dgemm_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.dgemm_q *  8) - 15) & ~15);
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_r = (((BUFFER_SIZE -
@@ -1049,26 +1803,33 @@ static void init_parameter(void) {
 			       ) / (TABLE_NAME.qgemm_q * 16) - 15) & ~15);
 #endif
 
+#if BUILD_COMPLEX ==1 
   TABLE_NAME.cgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.cgemm_p * TABLE_NAME.cgemm_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.cgemm_q *  8) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX16 ==1
   TABLE_NAME.zgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.zgemm_p * TABLE_NAME.zgemm_q * 16 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.zgemm_q * 16) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX == 1
   TABLE_NAME.cgemm3m_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.cgemm3m_p * TABLE_NAME.cgemm3m_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.cgemm3m_q *  8) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX16 == 1
   TABLE_NAME.zgemm3m_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.zgemm3m_p * TABLE_NAME.zgemm3m_q * 16 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.zgemm3m_q * 16) - 15) & ~15);
-
+#endif
 
 
 
@@ -1088,3 +1849,7 @@ static void init_parameter(void) {
 
 
 }
+#endif //POWER
+#endif //ZARCH
+#endif //(ARCH_MIPS64)
+#endif //(ARCH_ARM64)
