@@ -53,36 +53,33 @@ gotoblas_t TABLE_NAME = {
 
   GEMM_DEFAULT_OFFSET_A, GEMM_DEFAULT_OFFSET_B, GEMM_DEFAULT_ALIGN,
 
+#ifdef BUILD_BFLOAT16
   0, 0, 0,
-  SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N,
-#ifdef SGEMM_DEFAULT_UNROLL_MN
- SGEMM_DEFAULT_UNROLL_MN,
+  SBGEMM_DEFAULT_UNROLL_M, SBGEMM_DEFAULT_UNROLL_N,
+#ifdef SBGEMM_DEFAULT_UNROLL_MN
+ SBGEMM_DEFAULT_UNROLL_MN,
 #else
- MAX(SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N),
+ MAX(SBGEMM_DEFAULT_UNROLL_M, SBGEMM_DEFAULT_UNROLL_N),
 #endif
 
-
-#ifdef HAVE_EXCLUSIVE_CACHE
-  1,
-#else
-  0,
-#endif
+  sbstobf16_k_HASWELL, sbdtobf16_k_HASWELL, sbf16tos_k_HASWELL, dbf16tod_k_HASWELL,
 
   samax_k_HASWELL,  samin_k_HASWELL,  smax_k_HASWELL,  smin_k_HASWELL,
   isamax_k_HASWELL, isamin_k_HASWELL, ismax_k_HASWELL, ismin_k_HASWELL,
-  snrm2_k_HASWELL,  sasum_k_HASWELL,  scopy_k_HASWELL, sdot_k_HASWELL,
+  snrm2_k_HASWELL,  sasum_k_HASWELL, ssum_k_HASWELL, scopy_k_HASWELL, sbdot_k_HASWELL,
   dsdot_k_HASWELL,
   srot_k_HASWELL,   saxpy_k_HASWELL,  sscal_k_HASWELL, sswap_k_HASWELL,
-  sgemv_n_HASWELL,  sgemv_t_HASWELL, sger_k_HASWELL,
+  sbgemv_n_HASWELL, sbgemv_t_HASWELL, sger_k_HASWELL,
   ssymv_L_HASWELL, ssymv_U_HASWELL,
 
-  sgemm_kernel_HASWELL, sgemm_beta_HASWELL,
-#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
-  sgemm_incopy_HASWELL, sgemm_itcopy_HASWELL,
+  sbgemm_kernel_HASWELL, sbgemm_beta_HASWELL,
+#if SBGEMM_DEFAULT_UNROLL_M != SBGEMM_DEFAULT_UNROLL_N
+  sbgemm_incopy_HASWELL, sbgemm_itcopy_HASWELL,
 #else
-  sgemm_oncopy_HASWELL, sgemm_otcopy_HASWELL,
+  sbgemm_oncopy_HASWELL, sbgemm_otcopy_HASWELL,
 #endif
-  sgemm_oncopy_HASWELL, sgemm_otcopy_HASWELL,
+  sbgemm_oncopy_HASWELL, sbgemm_otcopy_HASWELL,
+
   strsm_kernel_LN_HASWELL, strsm_kernel_LT_HASWELL, strsm_kernel_RN_HASWELL, strsm_kernel_RT_HASWELL,
 #if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
   strsm_iunucopy_HASWELL, strsm_iunncopy_HASWELL, strsm_iutucopy_HASWELL, strsm_iutncopy_HASWELL,
@@ -115,7 +112,104 @@ gotoblas_t TABLE_NAME = {
 #else
   NULL,NULL,
 #endif
+#endif
 
+#if ( BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1) || (BUILD_COMPLEX16==1)
+  0, 0, 0,
+  SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N,
+#ifdef SGEMM_DEFAULT_UNROLL_MN
+ SGEMM_DEFAULT_UNROLL_MN,
+#else
+ MAX(SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N),
+#endif
+#endif
+
+#ifdef HAVE_EXCLUSIVE_CACHE
+  1,
+#else
+  0,
+#endif
+
+#if (BUILD_SINGLE==1 ) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  samax_k_HASWELL,  samin_k_HASWELL,  smax_k_HASWELL,  smin_k_HASWELL,
+  isamax_k_HASWELL, isamin_k_HASWELL, ismax_k_HASWELL, ismin_k_HASWELL,
+  snrm2_k_HASWELL,  sasum_k_HASWELL,
+#endif 
+#if BUILD_SINGLE == 1  
+  ssum_k_HASWELL,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  scopy_k_HASWELL, sdot_k_HASWELL,
+//  dsdot_k_HASWELL,
+  srot_k_HASWELL,   saxpy_k_HASWELL,  
+#endif
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1) || (BUILD_COMPLEX16==1)
+  sscal_k_HASWELL,
+#endif 
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  sswap_k_HASWELL,
+  sgemv_n_HASWELL,  sgemv_t_HASWELL,
+#endif
+#if BUILD_SINGLE == 1  
+  sger_k_HASWELL,
+  ssymv_L_HASWELL, ssymv_U_HASWELL,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+#ifdef ARCH_X86_64
+  sgemm_direct_HASWELL,
+  sgemm_direct_performant_HASWELL,	
+#endif
+
+  sgemm_kernel_HASWELL, sgemm_beta_HASWELL,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  sgemm_incopy_HASWELL, sgemm_itcopy_HASWELL,
+#else
+  sgemm_oncopy_HASWELL, sgemm_otcopy_HASWELL,
+#endif
+  sgemm_oncopy_HASWELL, sgemm_otcopy_HASWELL,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) 
+  strsm_kernel_LN_HASWELL, strsm_kernel_LT_HASWELL, strsm_kernel_RN_HASWELL, strsm_kernel_RT_HASWELL,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  strsm_iunucopy_HASWELL, strsm_iunncopy_HASWELL, strsm_iutucopy_HASWELL, strsm_iutncopy_HASWELL,
+  strsm_ilnucopy_HASWELL, strsm_ilnncopy_HASWELL, strsm_iltucopy_HASWELL, strsm_iltncopy_HASWELL,
+#else
+  strsm_ounucopy_HASWELL, strsm_ounncopy_HASWELL, strsm_outucopy_HASWELL, strsm_outncopy_HASWELL,
+  strsm_olnucopy_HASWELL, strsm_olnncopy_HASWELL, strsm_oltucopy_HASWELL, strsm_oltncopy_HASWELL,
+#endif
+  strsm_ounucopy_HASWELL, strsm_ounncopy_HASWELL, strsm_outucopy_HASWELL, strsm_outncopy_HASWELL,
+  strsm_olnucopy_HASWELL, strsm_olnncopy_HASWELL, strsm_oltucopy_HASWELL, strsm_oltncopy_HASWELL,
+#endif
+#if BUILD_SINGLE == 1
+  strmm_kernel_RN_HASWELL, strmm_kernel_RT_HASWELL, strmm_kernel_LN_HASWELL, strmm_kernel_LT_HASWELL,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  strmm_iunucopy_HASWELL, strmm_iunncopy_HASWELL, strmm_iutucopy_HASWELL, strmm_iutncopy_HASWELL,
+  strmm_ilnucopy_HASWELL, strmm_ilnncopy_HASWELL, strmm_iltucopy_HASWELL, strmm_iltncopy_HASWELL,
+#else
+  strmm_ounucopy_HASWELL, strmm_ounncopy_HASWELL, strmm_outucopy_HASWELL, strmm_outncopy_HASWELL,
+  strmm_olnucopy_HASWELL, strmm_olnncopy_HASWELL, strmm_oltucopy_HASWELL, strmm_oltncopy_HASWELL,
+#endif
+  strmm_ounucopy_HASWELL, strmm_ounncopy_HASWELL, strmm_outucopy_HASWELL, strmm_outncopy_HASWELL,
+  strmm_olnucopy_HASWELL, strmm_olnncopy_HASWELL, strmm_oltucopy_HASWELL, strmm_oltncopy_HASWELL,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  ssymm_iutcopy_HASWELL, ssymm_iltcopy_HASWELL,
+#else
+  ssymm_outcopy_HASWELL, ssymm_oltcopy_HASWELL,
+#endif
+  ssymm_outcopy_HASWELL, ssymm_oltcopy_HASWELL,
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_DOUBLE==1)
+#ifndef NO_LAPACK
+  sneg_tcopy_HASWELL, slaswp_ncopy_HASWELL,
+#else
+  NULL,NULL,
+#endif
+#endif
+
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   0, 0, 0,
   DGEMM_DEFAULT_UNROLL_M, DGEMM_DEFAULT_UNROLL_N,
 #ifdef DGEMM_DEFAULT_UNROLL_MN
@@ -123,14 +217,36 @@ gotoblas_t TABLE_NAME = {
 #else
  MAX(DGEMM_DEFAULT_UNROLL_M, DGEMM_DEFAULT_UNROLL_N),
 #endif
+#endif
 
+
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   damax_k_HASWELL,  damin_k_HASWELL,  dmax_k_HASWELL,  dmin_k_HASWELL,
   idamax_k_HASWELL, idamin_k_HASWELL, idmax_k_HASWELL, idmin_k_HASWELL,
-  dnrm2_k_HASWELL,  dasum_k_HASWELL,  dcopy_k_HASWELL, ddot_k_HASWELL,
-  drot_k_HASWELL,   daxpy_k_HASWELL,  dscal_k_HASWELL, dswap_k_HASWELL,
-  dgemv_n_HASWELL,  dgemv_t_HASWELL,  dger_k_HASWELL,
+  dnrm2_k_HASWELL, dasum_k_HASWELL,
+#endif  
+#if  (BUILD_DOUBLE==1)  
+  dsum_k_HASWELL,
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
+  dcopy_k_HASWELL, ddot_k_HASWELL,
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_DOUBLE==1)  
+  dsdot_k_HASWELL,
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
+  drot_k_HASWELL,
+  daxpy_k_HASWELL,
+  dscal_k_HASWELL, 
+  dswap_k_HASWELL,
+  dgemv_n_HASWELL,  dgemv_t_HASWELL,
+#endif
+#if  (BUILD_DOUBLE==1)  
+  dger_k_HASWELL,
   dsymv_L_HASWELL,  dsymv_U_HASWELL,
+#endif
 
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   dgemm_kernel_HASWELL, dgemm_beta_HASWELL,
 #if DGEMM_DEFAULT_UNROLL_M != DGEMM_DEFAULT_UNROLL_N
   dgemm_incopy_HASWELL, dgemm_itcopy_HASWELL,
@@ -138,6 +254,9 @@ gotoblas_t TABLE_NAME = {
   dgemm_oncopy_HASWELL, dgemm_otcopy_HASWELL,
 #endif
   dgemm_oncopy_HASWELL, dgemm_otcopy_HASWELL,
+#endif
+
+#if  (BUILD_DOUBLE==1)  
   dtrsm_kernel_LN_HASWELL, dtrsm_kernel_LT_HASWELL, dtrsm_kernel_RN_HASWELL, dtrsm_kernel_RT_HASWELL,
 #if DGEMM_DEFAULT_UNROLL_M != DGEMM_DEFAULT_UNROLL_N
   dtrsm_iunucopy_HASWELL, dtrsm_iunncopy_HASWELL, dtrsm_iutucopy_HASWELL, dtrsm_iutncopy_HASWELL,
@@ -171,6 +290,8 @@ gotoblas_t TABLE_NAME = {
   NULL, NULL,
 #endif
 
+#endif
+
 #ifdef EXPRECISION
 
   0, 0, 0,
@@ -178,7 +299,7 @@ gotoblas_t TABLE_NAME = {
 
   qamax_k_HASWELL,  qamin_k_HASWELL,  qmax_k_HASWELL,  qmin_k_HASWELL,
   iqamax_k_HASWELL, iqamin_k_HASWELL, iqmax_k_HASWELL, iqmin_k_HASWELL,
-  qnrm2_k_HASWELL,  qasum_k_HASWELL,  qcopy_k_HASWELL, qdot_k_HASWELL,
+  qnrm2_k_HASWELL,  qasum_k_HASWELL,  qsum_k_HASWELL, qcopy_k_HASWELL, qdot_k_HASWELL,
   qrot_k_HASWELL,   qaxpy_k_HASWELL,  qscal_k_HASWELL, qswap_k_HASWELL,
   qgemv_n_HASWELL,  qgemv_t_HASWELL,  qger_k_HASWELL,
   qsymv_L_HASWELL,  qsymv_U_HASWELL,
@@ -225,6 +346,7 @@ gotoblas_t TABLE_NAME = {
 
 #endif
 
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
   0, 0, 0,
   CGEMM_DEFAULT_UNROLL_M, CGEMM_DEFAULT_UNROLL_N,
 #ifdef CGEMM_DEFAULT_UNROLL_MN
@@ -232,21 +354,34 @@ gotoblas_t TABLE_NAME = {
 #else
  MAX(CGEMM_DEFAULT_UNROLL_M, CGEMM_DEFAULT_UNROLL_N),
 #endif
-
   camax_k_HASWELL, camin_k_HASWELL, icamax_k_HASWELL, icamin_k_HASWELL,
-  cnrm2_k_HASWELL, casum_k_HASWELL, ccopy_k_HASWELL,
-  cdotu_k_HASWELL, cdotc_k_HASWELL, csrot_k_HASWELL,
-  caxpy_k_HASWELL, caxpyc_k_HASWELL, cscal_k_HASWELL, cswap_k_HASWELL,
+#endif
+#if (BUILD_COMPLEX)
+  cnrm2_k_HASWELL, casum_k_HASWELL, csum_k_HASWELL,
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
+  ccopy_k_HASWELL,  cdotu_k_HASWELL, cdotc_k_HASWELL,
+#endif
+#if (BUILD_COMPLEX)
+ csrot_k_HASWELL,
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
+  caxpy_k_HASWELL,
+  caxpyc_k_HASWELL, 
+  cscal_k_HASWELL, 
+  cswap_k_HASWELL,
 
   cgemv_n_HASWELL, cgemv_t_HASWELL, cgemv_r_HASWELL, cgemv_c_HASWELL,
   cgemv_o_HASWELL, cgemv_u_HASWELL, cgemv_s_HASWELL, cgemv_d_HASWELL,
+#endif
+#if (BUILD_COMPLEX)
   cgeru_k_HASWELL, cgerc_k_HASWELL, cgerv_k_HASWELL, cgerd_k_HASWELL,
   csymv_L_HASWELL, csymv_U_HASWELL,
   chemv_L_HASWELL, chemv_U_HASWELL, chemv_M_HASWELL, chemv_V_HASWELL,
-
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
   cgemm_kernel_n_HASWELL, cgemm_kernel_l_HASWELL, cgemm_kernel_r_HASWELL, cgemm_kernel_b_HASWELL,
   cgemm_beta_HASWELL,
-
 #if CGEMM_DEFAULT_UNROLL_M != CGEMM_DEFAULT_UNROLL_N
   cgemm_incopy_HASWELL, cgemm_itcopy_HASWELL,
 #else
@@ -266,6 +401,8 @@ gotoblas_t TABLE_NAME = {
 #endif
   ctrsm_ounucopy_HASWELL,  ctrsm_ounncopy_HASWELL,  ctrsm_outucopy_HASWELL,  ctrsm_outncopy_HASWELL,
   ctrsm_olnucopy_HASWELL,  ctrsm_olnncopy_HASWELL,  ctrsm_oltucopy_HASWELL,  ctrsm_oltncopy_HASWELL,
+#endif
+#if (BUILD_COMPLEX)
 
   ctrmm_kernel_RN_HASWELL,  ctrmm_kernel_RT_HASWELL,  ctrmm_kernel_RR_HASWELL,  ctrmm_kernel_RC_HASWELL,
   ctrmm_kernel_LN_HASWELL,  ctrmm_kernel_LT_HASWELL,  ctrmm_kernel_LR_HASWELL,  ctrmm_kernel_LC_HASWELL,
@@ -294,6 +431,8 @@ gotoblas_t TABLE_NAME = {
   chemm_outcopy_HASWELL,  chemm_oltcopy_HASWELL,
 
   0, 0, 0,
+
+#if (USE_GEMM3M)
 #ifdef CGEMM3M_DEFAULT_UNROLL_M
   CGEMM3M_DEFAULT_UNROLL_M, CGEMM3M_DEFAULT_UNROLL_N, MAX(CGEMM3M_DEFAULT_UNROLL_M, CGEMM3M_DEFAULT_UNROLL_N),
 #else
@@ -324,13 +463,47 @@ gotoblas_t TABLE_NAME = {
   chemm3m_oucopyb_HASWELL,  chemm3m_olcopyb_HASWELL,
   chemm3m_oucopyr_HASWELL,  chemm3m_olcopyr_HASWELL,
   chemm3m_oucopyi_HASWELL,  chemm3m_olcopyi_HASWELL,
+#else
+  0, 0, 0,
 
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
+#endif
+
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
 #ifndef NO_LAPACK
-  cneg_tcopy_HASWELL, claswp_ncopy_HASWELL,
+  cneg_tcopy_HASWELL,
+  
+   claswp_ncopy_HASWELL,
 #else
   NULL, NULL,
 #endif
 
+#endif
+
+#if BUILD_COMPLEX16 == 1
   0, 0, 0,
   ZGEMM_DEFAULT_UNROLL_M, ZGEMM_DEFAULT_UNROLL_N,
 #ifdef ZGEMM_DEFAULT_UNROLL_MN
@@ -340,7 +513,7 @@ gotoblas_t TABLE_NAME = {
 #endif
 
   zamax_k_HASWELL, zamin_k_HASWELL, izamax_k_HASWELL, izamin_k_HASWELL,
-  znrm2_k_HASWELL, zasum_k_HASWELL, zcopy_k_HASWELL,
+  znrm2_k_HASWELL, zasum_k_HASWELL, zsum_k_HASWELL, zcopy_k_HASWELL,
   zdotu_k_HASWELL, zdotc_k_HASWELL, zdrot_k_HASWELL,
   zaxpy_k_HASWELL, zaxpyc_k_HASWELL, zscal_k_HASWELL, zswap_k_HASWELL,
 
@@ -400,6 +573,7 @@ gotoblas_t TABLE_NAME = {
   zhemm_outcopy_HASWELL,  zhemm_oltcopy_HASWELL,
 
   0, 0, 0,
+#if (USE_GEMM3M)
 #ifdef ZGEMM3M_DEFAULT_UNROLL_M
   ZGEMM3M_DEFAULT_UNROLL_M, ZGEMM3M_DEFAULT_UNROLL_N, MAX(ZGEMM3M_DEFAULT_UNROLL_M, ZGEMM3M_DEFAULT_UNROLL_N),
 #else
@@ -430,6 +604,33 @@ gotoblas_t TABLE_NAME = {
   zhemm3m_oucopyb_HASWELL,  zhemm3m_olcopyb_HASWELL,
   zhemm3m_oucopyr_HASWELL,  zhemm3m_olcopyr_HASWELL,
   zhemm3m_oucopyi_HASWELL,  zhemm3m_olcopyi_HASWELL,
+#else
+  0, 0, 0,
+
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
 
 #ifndef NO_LAPACK
   zneg_tcopy_HASWELL, zlaswp_ncopy_HASWELL,
@@ -437,13 +638,15 @@ gotoblas_t TABLE_NAME = {
   NULL, NULL,
 #endif
 
+#endif
+
 #ifdef EXPRECISION
 
   0, 0, 0,
   XGEMM_DEFAULT_UNROLL_M, XGEMM_DEFAULT_UNROLL_N, MAX(XGEMM_DEFAULT_UNROLL_M, XGEMM_DEFAULT_UNROLL_N),
 
   xamax_k_HASWELL, xamin_k_HASWELL, ixamax_k_HASWELL, ixamin_k_HASWELL,
-  xnrm2_k_HASWELL, xasum_k_HASWELL, xcopy_k_HASWELL,
+  xnrm2_k_HASWELL, xasum_k_HASWELL, xsum_k_HASWELL, xcopy_k_HASWELL,
   xdotu_k_HASWELL, xdotc_k_HASWELL, xqrot_k_HASWELL,
   xaxpy_k_HASWELL, xaxpyc_k_HASWELL, xscal_k_HASWELL, xswap_k_HASWELL,
 
@@ -503,6 +706,7 @@ gotoblas_t TABLE_NAME = {
   xhemm_outcopy_HASWELL,  xhemm_oltcopy_HASWELL,
 
   0, 0, 0,
+#if (USE_GEMM3M)
   QGEMM_DEFAULT_UNROLL_M, QGEMM_DEFAULT_UNROLL_N, MAX(QGEMM_DEFAULT_UNROLL_M, QGEMM_DEFAULT_UNROLL_N),
 
   xgemm3m_kernel_HASWELL,
@@ -528,6 +732,33 @@ gotoblas_t TABLE_NAME = {
   xhemm3m_oucopyb_HASWELL,  xhemm3m_olcopyb_HASWELL,
   xhemm3m_oucopyr_HASWELL,  xhemm3m_olcopyr_HASWELL,
   xhemm3m_oucopyi_HASWELL,  xhemm3m_olcopyi_HASWELL,
+#else
+  0, 0, 0,
+
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
 
 #ifndef NO_LAPACK
   xneg_tcopy_HASWELL, xlaswp_ncopy_HASWELL,
@@ -540,27 +771,298 @@ gotoblas_t TABLE_NAME = {
   init_parameter,
 
   SNUMOPT, DNUMOPT, QNUMOPT,
+#if BUILD_SINGLE == 1
+  saxpby_k_HASWELL,
+#endif
+#if BUILD_DOUBLE  == 1
+  daxpby_k_HASWELL,
+#endif
+#if BUILD_COMPLEX == 1
+  caxpby_k_HASWELL,
+#endif
+#if BUILD_COMPLEX16== 1
+  zaxpby_k_HASWELL,
+#endif
 
-  saxpby_k_HASWELL, daxpby_k_HASWELL, caxpby_k_HASWELL, zaxpby_k_HASWELL,
-
+#if BUILD_SINGLE == 1
   somatcopy_k_cn_HASWELL, somatcopy_k_ct_HASWELL, somatcopy_k_rn_HASWELL, somatcopy_k_rt_HASWELL,
+#endif
+#if BUILD_DOUBLE== 1
   domatcopy_k_cn_HASWELL, domatcopy_k_ct_HASWELL, domatcopy_k_rn_HASWELL, domatcopy_k_rt_HASWELL,
+#endif
+#if BUILD_COMPLEX == 1
   comatcopy_k_cn_HASWELL, comatcopy_k_ct_HASWELL, comatcopy_k_rn_HASWELL, comatcopy_k_rt_HASWELL,
   comatcopy_k_cnc_HASWELL, comatcopy_k_ctc_HASWELL, comatcopy_k_rnc_HASWELL, comatcopy_k_rtc_HASWELL,
+#endif
+#if BUILD_COMPLEX16 == 1
   zomatcopy_k_cn_HASWELL, zomatcopy_k_ct_HASWELL, zomatcopy_k_rn_HASWELL, zomatcopy_k_rt_HASWELL,
   zomatcopy_k_cnc_HASWELL, zomatcopy_k_ctc_HASWELL, zomatcopy_k_rnc_HASWELL, zomatcopy_k_rtc_HASWELL,
+#endif
 
+#if BUILD_SINGLE == 1
   simatcopy_k_cn_HASWELL, simatcopy_k_ct_HASWELL, simatcopy_k_rn_HASWELL, simatcopy_k_rt_HASWELL,
+#endif
+#if BUILD_DOUBLE== 1
   dimatcopy_k_cn_HASWELL, dimatcopy_k_ct_HASWELL, dimatcopy_k_rn_HASWELL, dimatcopy_k_rt_HASWELL,
+#endif
+#if BUILD_COMPLEX== 1
   cimatcopy_k_cn_HASWELL, cimatcopy_k_ct_HASWELL, cimatcopy_k_rn_HASWELL, cimatcopy_k_rt_HASWELL,
   cimatcopy_k_cnc_HASWELL, cimatcopy_k_ctc_HASWELL, cimatcopy_k_rnc_HASWELL, cimatcopy_k_rtc_HASWELL,
+#endif
+#if BUILD_COMPLEX16==1
   zimatcopy_k_cn_HASWELL, zimatcopy_k_ct_HASWELL, zimatcopy_k_rn_HASWELL, zimatcopy_k_rt_HASWELL,
   zimatcopy_k_cnc_HASWELL, zimatcopy_k_ctc_HASWELL, zimatcopy_k_rnc_HASWELL, zimatcopy_k_rtc_HASWELL,
+#endif
 
-  sgeadd_k_HASWELL, dgeadd_k_HASWELL, cgeadd_k_HASWELL, zgeadd_k_HASWELL
-
+#if BUILD_SINGLE == 1
+  sgeadd_k_HASWELL,
+#endif
+#if BUILD_DOUBLE==1
+  dgeadd_k_HASWELL,
+#endif
+#if BUILD_COMPLEX==1
+  cgeadd_k_HASWELL,
+#endif
+#if BUILD_COMPLEX16==1
+  zgeadd_k_HASWELL
+#endif
 };
 
+#if (ARCH_ARM64)
+static void init_parameter(void) {
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+#if (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE == 1
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+#endif
+#if BUILD_DOUBLE== 1
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX== 1
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+#endif
+
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+#endif
+#if BUILD_DOUBLE==1 
+  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+  TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
+  TABLE_NAME.xgemm_q = XGEMM_DEFAULT_Q;
+  TABLE_NAME.qgemm_r = QGEMM_DEFAULT_R;
+  TABLE_NAME.xgemm_r = XGEMM_DEFAULT_R;
+#endif
+
+#if (USE_GEMM3M)
+#ifdef CGEMM3M_DEFAULT_P
+  TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_P
+  TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_Q
+  TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.cgemm3m_q = TABLE_NAME.sgemm_q;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_Q
+  TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.zgemm3m_q = TABLE_NAME.dgemm_q;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_R
+  TABLE_NAME.cgemm3m_r = CGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.cgemm3m_r = TABLE_NAME.sgemm_r;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_R
+  TABLE_NAME.zgemm3m_r = ZGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.zgemm3m_r = TABLE_NAME.dgemm_r;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
+  TABLE_NAME.xgemm3m_q = TABLE_NAME.qgemm_q;
+  TABLE_NAME.xgemm3m_r = TABLE_NAME.qgemm_r;
+#endif
+#endif
+
+}
+#else // (ARCH_ARM64)
+#if defined(ARCH_MIPS64)
+static void init_parameter(void) {
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+  TABLE_NAME.dgemm_r = 640;
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+  TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
+  TABLE_NAME.xgemm_q = XGEMM_DEFAULT_Q;
+  TABLE_NAME.qgemm_r = QGEMM_DEFAULT_R;
+  TABLE_NAME.xgemm_r = XGEMM_DEFAULT_R;
+#endif
+
+#if defined(USE_GEMM3M)
+#ifdef CGEMM3M_DEFAULT_P
+  TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_P
+  TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_Q
+  TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.cgemm3m_q = TABLE_NAME.sgemm_q;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_Q
+  TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.zgemm3m_q = TABLE_NAME.dgemm_q;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_R
+  TABLE_NAME.cgemm3m_r = CGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.cgemm3m_r = TABLE_NAME.sgemm_r;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_R
+  TABLE_NAME.zgemm3m_r = ZGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.zgemm3m_r = TABLE_NAME.dgemm_r;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
+  TABLE_NAME.xgemm3m_q = TABLE_NAME.qgemm_q;
+  TABLE_NAME.xgemm3m_r = TABLE_NAME.qgemm_r;
+#endif
+#endif
+}
+#else // (ARCH_MIPS64)
+#if (ARCH_POWER)
+static void init_parameter(void) {
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+}
+#else //POWER
+
+#if (ARCH_ZARCH)
+static void init_parameter(void) {
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+	TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+	TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+	TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+	TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+	TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+	TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+	TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+	TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+	TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+	TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+	TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+	TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+}
+#else //ZARCH
+
 #ifdef ARCH_X86
 static int get_l2_size_old(void){
   int i, eax, ebx, ecx, edx, cpuid_level;
@@ -647,7 +1149,9 @@ static int get_l2_size_old(void){
       return 6144;
     }
   }
-  return 0;
+//  return 0;
+fprintf (stderr,"OpenBLAS WARNING - could not determine the L2 cache size on this system, assuming 256k\n");
+return 256;
 }
 #endif
 
@@ -660,6 +1164,10 @@ static __inline__ int get_l2_size(void){
   l2 = BITMASK(ecx, 16, 0xffff);
 
 #ifndef ARCH_X86
+  if (l2 <= 0) {
+     fprintf (stderr,"OpenBLAS WARNING - could not determine the L2 cache size on this system, assuming 256k\n");
+     return 256;
+  }
   return l2;
 
 #else
@@ -684,22 +1192,42 @@ static void init_parameter(void) {
 
   int l2 = get_l2_size();
 
+  (void) l2; /* dirty trick to suppress unused variable warning for targets */
+             /* where the GEMM unrolling parameters do not depend on l2 */
+  
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)
   TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX == 1
   TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+#endif
 
+#if BUILD_COMPLEX == 1
 #ifdef CGEMM3M_DEFAULT_Q
   TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
 #else
   TABLE_NAME.cgemm3m_q = SGEMM_DEFAULT_Q;
 #endif
+#endif
 
+#if BUILD_COMPLEX16 == 1
 #ifdef ZGEMM3M_DEFAULT_Q
   TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
 #else
   TABLE_NAME.zgemm3m_q = DGEMM_DEFAULT_Q;
 #endif
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
@@ -713,10 +1241,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Katmai, Coppermine, Banias, Athlon\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  64 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  32 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1 
   TABLE_NAME.cgemm_p =  32 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  16 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  16 * (l2 >> 7);
   TABLE_NAME.xgemm_p =   8 * (l2 >> 7);
@@ -729,10 +1265,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Northwood\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  96 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  48 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1 
   TABLE_NAME.cgemm_p =  48 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  24 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  24 * (l2 >> 7);
   TABLE_NAME.xgemm_p =  12 * (l2 >> 7);
@@ -745,10 +1289,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Atom\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = 256;
+#endif
+#if BUILD_DOUBLE ==1
   TABLE_NAME.dgemm_p = 128;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p = 128;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  64;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  64;
   TABLE_NAME.xgemm_p =  32;
@@ -761,10 +1313,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Prescott\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  56 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE ==1 
   TABLE_NAME.dgemm_p =  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16 == 1
   TABLE_NAME.zgemm_p =  14 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  14 * (l2 >> 7);
   TABLE_NAME.xgemm_p =   7 * (l2 >> 7);
@@ -777,10 +1337,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Core2\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  92 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE==1
   TABLE_NAME.dgemm_p =  46 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  46 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  23 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  92 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  46 * (l2 >> 9) + 4;
@@ -793,10 +1361,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Penryn\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  21 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  21 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  42 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  21 * (l2 >> 9) + 4;
@@ -809,10 +1385,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Dunnington\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE ==1
   TABLE_NAME.dgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  21 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  21 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  42 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  21 * (l2 >> 9) + 4;
@@ -826,10 +1410,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Nehalem\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -842,10 +1434,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Sandybridge\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -858,10 +1458,42 @@ static void init_parameter(void) {
   fprintf(stderr, "Haswell\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+#if defined(SKYLAKEX) || defined(COOPERLAKE)
+
+#ifdef DEBUG
+  fprintf(stderr, "SkylakeX\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -875,10 +1507,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Opteron\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = 224 +  56 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = 112 +  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = 112 +  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p =  56 +  14 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  56 +  14 * (l2 >> 7);
   TABLE_NAME.xgemm_p =  28 +   7 * (l2 >> 7);
@@ -891,10 +1531,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Barcelona\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -907,10 +1555,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Bobcate\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -923,26 +1579,67 @@ static void init_parameter(void) {
   fprintf(stderr, "Bulldozer\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
 #endif
 #endif
 
+#ifdef EXCAVATOR
+
+#ifdef DEBUG
+  fprintf(stderr, "Excavator\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+
 #ifdef PILEDRIVER
 
 #ifdef DEBUG
   fprintf(stderr, "Piledriver\n");
 #endif
 
+#if (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -955,10 +1652,42 @@ static void init_parameter(void) {
   fprintf(stderr, "Steamroller\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+#ifdef ZEN
+
+#ifdef DEBUG
+  fprintf(stderr, "Zen\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -972,11 +1701,18 @@ static void init_parameter(void) {
   fprintf(stderr, "NANO\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if  (BUILD_DOUBLE==1)
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if (BUILD_COMPLEX==1)
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if (BUILD_COMPLEX16==1)
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
-
+#endif
 
 
 #ifdef EXPRECISION
@@ -986,61 +1722,79 @@ static void init_parameter(void) {
 
 #endif
 
-
+#if BUILD_COMPLEX==1
 #ifdef CGEMM3M_DEFAULT_P
   TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
 #else
   TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
 #endif
+#endif
 
+#if BUILD_COMPLEX16==1
 #ifdef ZGEMM3M_DEFAULT_P
   TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
 #else
   TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
 #endif
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
 #endif
 
 
-
-  TABLE_NAME.sgemm_p = (TABLE_NAME.sgemm_p + SGEMM_DEFAULT_UNROLL_M - 1) & ~(SGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.dgemm_p = (TABLE_NAME.dgemm_p + DGEMM_DEFAULT_UNROLL_M - 1) & ~(DGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.cgemm_p = (TABLE_NAME.cgemm_p + CGEMM_DEFAULT_UNROLL_M - 1) & ~(CGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.zgemm_p = (TABLE_NAME.zgemm_p + ZGEMM_DEFAULT_UNROLL_M - 1) & ~(ZGEMM_DEFAULT_UNROLL_M - 1);
-
-#ifdef CGEMM3M_DEFAULT_UNROLL_M
-  TABLE_NAME.cgemm3m_p = (TABLE_NAME.cgemm3m_p + CGEMM3M_DEFAULT_UNROLL_M - 1) & ~(CGEMM3M_DEFAULT_UNROLL_M - 1);
-#else
-  TABLE_NAME.cgemm3m_p = (TABLE_NAME.cgemm3m_p + SGEMM_DEFAULT_UNROLL_M - 1) & ~(SGEMM_DEFAULT_UNROLL_M - 1);
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_p = ((TABLE_NAME.sgemm_p + SGEMM_DEFAULT_UNROLL_M - 1)/SGEMM_DEFAULT_UNROLL_M) * SGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_DOUBLE== 1
+  TABLE_NAME.dgemm_p = ((TABLE_NAME.dgemm_p + DGEMM_DEFAULT_UNROLL_M - 1)/DGEMM_DEFAULT_UNROLL_M) * DGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_p = ((TABLE_NAME.cgemm_p + CGEMM_DEFAULT_UNROLL_M - 1)/CGEMM_DEFAULT_UNROLL_M) * CGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_p = ((TABLE_NAME.zgemm_p + ZGEMM_DEFAULT_UNROLL_M - 1)/ZGEMM_DEFAULT_UNROLL_M) * ZGEMM_DEFAULT_UNROLL_M;
 #endif
 
-#ifdef ZGEMM3M_DEFAULT_UNROLL_M
-  TABLE_NAME.zgemm3m_p = (TABLE_NAME.zgemm3m_p + ZGEMM3M_DEFAULT_UNROLL_M - 1) & ~(ZGEMM3M_DEFAULT_UNROLL_M - 1);
+#if BUILD_COMPLEX==1
+#ifdef CGEMM3M_DEFAULT_UNROLL_M
+  TABLE_NAME.cgemm3m_p = ((TABLE_NAME.cgemm3m_p + CGEMM3M_DEFAULT_UNROLL_M - 1)/CGEMM3M_DEFAULT_UNROLL_M) * CGEMM3M_DEFAULT_UNROLL_M;
 #else
-  TABLE_NAME.zgemm3m_p = (TABLE_NAME.zgemm3m_p + DGEMM_DEFAULT_UNROLL_M - 1) & ~(DGEMM_DEFAULT_UNROLL_M - 1);
+  TABLE_NAME.cgemm3m_p = ((TABLE_NAME.cgemm3m_p + SGEMM_DEFAULT_UNROLL_M - 1)/SGEMM_DEFAULT_UNROLL_M) * SGEMM_DEFAULT_UNROLL_M;
+#endif
+#endif
+
+#if BUILD_COMPLEX16==1
+#ifdef ZGEMM3M_DEFAULT_UNROLL_M
+  TABLE_NAME.zgemm3m_p = ((TABLE_NAME.zgemm3m_p + ZGEMM3M_DEFAULT_UNROLL_M - 1)/ZGEMM3M_DEFAULT_UNROLL_M) * ZGEMM3M_DEFAULT_UNROLL_M;
+#else
+  TABLE_NAME.zgemm3m_p = ((TABLE_NAME.zgemm3m_p + DGEMM_DEFAULT_UNROLL_M - 1)/DGEMM_DEFAULT_UNROLL_M) * DGEMM_DEFAULT_UNROLL_M;
+#endif
 #endif
 
 #ifdef QUAD_PRECISION
-  TABLE_NAME.qgemm_p = (TABLE_NAME.qgemm_p + QGEMM_DEFAULT_UNROLL_M - 1) & ~(QGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.xgemm_p = (TABLE_NAME.xgemm_p + XGEMM_DEFAULT_UNROLL_M - 1) & ~(XGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.xgemm3m_p = (TABLE_NAME.xgemm3m_p + QGEMM_DEFAULT_UNROLL_M - 1) & ~(QGEMM_DEFAULT_UNROLL_M - 1);
+  TABLE_NAME.qgemm_p = ((TABLE_NAME.qgemm_p + QGEMM_DEFAULT_UNROLL_M - 1)/QGEMM_DEFAULT_UNROLL_M) * QGEMM_DEFAULT_UNROLL_M;
+  TABLE_NAME.xgemm_p = ((TABLE_NAME.xgemm_p + XGEMM_DEFAULT_UNROLL_M - 1)/XGEMM_DEFAULT_UNROLL_M) * XGEMM_DEFAULT_UNROLL_M;
+  TABLE_NAME.xgemm3m_p = ((TABLE_NAME.xgemm3m_p + QGEMM_DEFAULT_UNROLL_M - 1)/QGEMM_DEFAULT_UNROLL_M) * QGEMM_DEFAULT_UNROLL_M;
 #endif
 
 #ifdef DEBUG
   fprintf(stderr, "L2 = %8d DGEMM_P  .. %d\n", l2, TABLE_NAME.dgemm_p);
 #endif
 
+#if BUILD_SINGLE==1
   TABLE_NAME.sgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.sgemm_p * TABLE_NAME.sgemm_q *  4 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.sgemm_q *  4) - 15) & ~15);
+#endif
 
+#if BUILD_DOUBLE==1
   TABLE_NAME.dgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.dgemm_p * TABLE_NAME.dgemm_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.dgemm_q *  8) - 15) & ~15);
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_r = (((BUFFER_SIZE -
@@ -1049,26 +1803,33 @@ static void init_parameter(void) {
 			       ) / (TABLE_NAME.qgemm_q * 16) - 15) & ~15);
 #endif
 
+#if BUILD_COMPLEX ==1 
   TABLE_NAME.cgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.cgemm_p * TABLE_NAME.cgemm_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.cgemm_q *  8) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX16 ==1
   TABLE_NAME.zgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.zgemm_p * TABLE_NAME.zgemm_q * 16 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.zgemm_q * 16) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX == 1
   TABLE_NAME.cgemm3m_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.cgemm3m_p * TABLE_NAME.cgemm3m_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.cgemm3m_q *  8) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX16 == 1
   TABLE_NAME.zgemm3m_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.zgemm3m_p * TABLE_NAME.zgemm3m_q * 16 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.zgemm3m_q * 16) - 15) & ~15);
-
+#endif
 
 
 
@@ -1088,3 +1849,7 @@ static void init_parameter(void) {
 
 
 }
+#endif //POWER
+#endif //ZARCH
+#endif //(ARCH_MIPS64)
+#endif //(ARCH_ARM64)
