@@ -53,36 +53,33 @@ gotoblas_t TABLE_NAME = {
 
   GEMM_DEFAULT_OFFSET_A, GEMM_DEFAULT_OFFSET_B, GEMM_DEFAULT_ALIGN,
 
+#ifdef BUILD_BFLOAT16
   0, 0, 0,
-  SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N,
-#ifdef SGEMM_DEFAULT_UNROLL_MN
- SGEMM_DEFAULT_UNROLL_MN,
+  SBGEMM_DEFAULT_UNROLL_M, SBGEMM_DEFAULT_UNROLL_N,
+#ifdef SBGEMM_DEFAULT_UNROLL_MN
+ SBGEMM_DEFAULT_UNROLL_MN,
 #else
- MAX(SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N),
+ MAX(SBGEMM_DEFAULT_UNROLL_M, SBGEMM_DEFAULT_UNROLL_N),
 #endif
 
-
-#ifdef HAVE_EXCLUSIVE_CACHE
-  1,
-#else
-  0,
-#endif
+  sbstobf16_k_NEHALEM, sbdtobf16_k_NEHALEM, sbf16tos_k_NEHALEM, dbf16tod_k_NEHALEM,
 
   samax_k_NEHALEM,  samin_k_NEHALEM,  smax_k_NEHALEM,  smin_k_NEHALEM,
   isamax_k_NEHALEM, isamin_k_NEHALEM, ismax_k_NEHALEM, ismin_k_NEHALEM,
-  snrm2_k_NEHALEM,  sasum_k_NEHALEM,  scopy_k_NEHALEM, sdot_k_NEHALEM,
+  snrm2_k_NEHALEM,  sasum_k_NEHALEM, ssum_k_NEHALEM, scopy_k_NEHALEM, sbdot_k_NEHALEM,
   dsdot_k_NEHALEM,
   srot_k_NEHALEM,   saxpy_k_NEHALEM,  sscal_k_NEHALEM, sswap_k_NEHALEM,
-  sgemv_n_NEHALEM,  sgemv_t_NEHALEM, sger_k_NEHALEM,
+  sbgemv_n_NEHALEM, sbgemv_t_NEHALEM, sger_k_NEHALEM,
   ssymv_L_NEHALEM, ssymv_U_NEHALEM,
 
-  sgemm_kernel_NEHALEM, sgemm_beta_NEHALEM,
-#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
-  sgemm_incopy_NEHALEM, sgemm_itcopy_NEHALEM,
+  sbgemm_kernel_NEHALEM, sbgemm_beta_NEHALEM,
+#if SBGEMM_DEFAULT_UNROLL_M != SBGEMM_DEFAULT_UNROLL_N
+  sbgemm_incopy_NEHALEM, sbgemm_itcopy_NEHALEM,
 #else
-  sgemm_oncopy_NEHALEM, sgemm_otcopy_NEHALEM,
+  sbgemm_oncopy_NEHALEM, sbgemm_otcopy_NEHALEM,
 #endif
-  sgemm_oncopy_NEHALEM, sgemm_otcopy_NEHALEM,
+  sbgemm_oncopy_NEHALEM, sbgemm_otcopy_NEHALEM,
+
   strsm_kernel_LN_NEHALEM, strsm_kernel_LT_NEHALEM, strsm_kernel_RN_NEHALEM, strsm_kernel_RT_NEHALEM,
 #if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
   strsm_iunucopy_NEHALEM, strsm_iunncopy_NEHALEM, strsm_iutucopy_NEHALEM, strsm_iutncopy_NEHALEM,
@@ -115,7 +112,104 @@ gotoblas_t TABLE_NAME = {
 #else
   NULL,NULL,
 #endif
+#endif
 
+#if ( BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1) || (BUILD_COMPLEX16==1)
+  0, 0, 0,
+  SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N,
+#ifdef SGEMM_DEFAULT_UNROLL_MN
+ SGEMM_DEFAULT_UNROLL_MN,
+#else
+ MAX(SGEMM_DEFAULT_UNROLL_M, SGEMM_DEFAULT_UNROLL_N),
+#endif
+#endif
+
+#ifdef HAVE_EXCLUSIVE_CACHE
+  1,
+#else
+  0,
+#endif
+
+#if (BUILD_SINGLE==1 ) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  samax_k_NEHALEM,  samin_k_NEHALEM,  smax_k_NEHALEM,  smin_k_NEHALEM,
+  isamax_k_NEHALEM, isamin_k_NEHALEM, ismax_k_NEHALEM, ismin_k_NEHALEM,
+  snrm2_k_NEHALEM,  sasum_k_NEHALEM,
+#endif 
+#if BUILD_SINGLE == 1  
+  ssum_k_NEHALEM,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  scopy_k_NEHALEM, sdot_k_NEHALEM,
+//  dsdot_k_NEHALEM,
+  srot_k_NEHALEM,   saxpy_k_NEHALEM,  
+#endif
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1) || (BUILD_COMPLEX16==1)
+  sscal_k_NEHALEM,
+#endif 
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+  sswap_k_NEHALEM,
+  sgemv_n_NEHALEM,  sgemv_t_NEHALEM,
+#endif
+#if BUILD_SINGLE == 1  
+  sger_k_NEHALEM,
+  ssymv_L_NEHALEM, ssymv_U_NEHALEM,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) || (BUILD_COMPLEX==1)
+#ifdef ARCH_X86_64
+  sgemm_direct_NEHALEM,
+  sgemm_direct_performant_NEHALEM,	
+#endif
+
+  sgemm_kernel_NEHALEM, sgemm_beta_NEHALEM,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  sgemm_incopy_NEHALEM, sgemm_itcopy_NEHALEM,
+#else
+  sgemm_oncopy_NEHALEM, sgemm_otcopy_NEHALEM,
+#endif
+  sgemm_oncopy_NEHALEM, sgemm_otcopy_NEHALEM,
+#endif
+
+#if (BUILD_SINGLE==1) || (BUILD_DOUBLE==1) 
+  strsm_kernel_LN_NEHALEM, strsm_kernel_LT_NEHALEM, strsm_kernel_RN_NEHALEM, strsm_kernel_RT_NEHALEM,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  strsm_iunucopy_NEHALEM, strsm_iunncopy_NEHALEM, strsm_iutucopy_NEHALEM, strsm_iutncopy_NEHALEM,
+  strsm_ilnucopy_NEHALEM, strsm_ilnncopy_NEHALEM, strsm_iltucopy_NEHALEM, strsm_iltncopy_NEHALEM,
+#else
+  strsm_ounucopy_NEHALEM, strsm_ounncopy_NEHALEM, strsm_outucopy_NEHALEM, strsm_outncopy_NEHALEM,
+  strsm_olnucopy_NEHALEM, strsm_olnncopy_NEHALEM, strsm_oltucopy_NEHALEM, strsm_oltncopy_NEHALEM,
+#endif
+  strsm_ounucopy_NEHALEM, strsm_ounncopy_NEHALEM, strsm_outucopy_NEHALEM, strsm_outncopy_NEHALEM,
+  strsm_olnucopy_NEHALEM, strsm_olnncopy_NEHALEM, strsm_oltucopy_NEHALEM, strsm_oltncopy_NEHALEM,
+#endif
+#if BUILD_SINGLE == 1
+  strmm_kernel_RN_NEHALEM, strmm_kernel_RT_NEHALEM, strmm_kernel_LN_NEHALEM, strmm_kernel_LT_NEHALEM,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  strmm_iunucopy_NEHALEM, strmm_iunncopy_NEHALEM, strmm_iutucopy_NEHALEM, strmm_iutncopy_NEHALEM,
+  strmm_ilnucopy_NEHALEM, strmm_ilnncopy_NEHALEM, strmm_iltucopy_NEHALEM, strmm_iltncopy_NEHALEM,
+#else
+  strmm_ounucopy_NEHALEM, strmm_ounncopy_NEHALEM, strmm_outucopy_NEHALEM, strmm_outncopy_NEHALEM,
+  strmm_olnucopy_NEHALEM, strmm_olnncopy_NEHALEM, strmm_oltucopy_NEHALEM, strmm_oltncopy_NEHALEM,
+#endif
+  strmm_ounucopy_NEHALEM, strmm_ounncopy_NEHALEM, strmm_outucopy_NEHALEM, strmm_outncopy_NEHALEM,
+  strmm_olnucopy_NEHALEM, strmm_olnncopy_NEHALEM, strmm_oltucopy_NEHALEM, strmm_oltncopy_NEHALEM,
+#if SGEMM_DEFAULT_UNROLL_M != SGEMM_DEFAULT_UNROLL_N
+  ssymm_iutcopy_NEHALEM, ssymm_iltcopy_NEHALEM,
+#else
+  ssymm_outcopy_NEHALEM, ssymm_oltcopy_NEHALEM,
+#endif
+  ssymm_outcopy_NEHALEM, ssymm_oltcopy_NEHALEM,
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_DOUBLE==1)
+#ifndef NO_LAPACK
+  sneg_tcopy_NEHALEM, slaswp_ncopy_NEHALEM,
+#else
+  NULL,NULL,
+#endif
+#endif
+
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   0, 0, 0,
   DGEMM_DEFAULT_UNROLL_M, DGEMM_DEFAULT_UNROLL_N,
 #ifdef DGEMM_DEFAULT_UNROLL_MN
@@ -123,14 +217,36 @@ gotoblas_t TABLE_NAME = {
 #else
  MAX(DGEMM_DEFAULT_UNROLL_M, DGEMM_DEFAULT_UNROLL_N),
 #endif
+#endif
 
+
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   damax_k_NEHALEM,  damin_k_NEHALEM,  dmax_k_NEHALEM,  dmin_k_NEHALEM,
   idamax_k_NEHALEM, idamin_k_NEHALEM, idmax_k_NEHALEM, idmin_k_NEHALEM,
-  dnrm2_k_NEHALEM,  dasum_k_NEHALEM,  dcopy_k_NEHALEM, ddot_k_NEHALEM,
-  drot_k_NEHALEM,   daxpy_k_NEHALEM,  dscal_k_NEHALEM, dswap_k_NEHALEM,
-  dgemv_n_NEHALEM,  dgemv_t_NEHALEM,  dger_k_NEHALEM,
+  dnrm2_k_NEHALEM, dasum_k_NEHALEM,
+#endif  
+#if  (BUILD_DOUBLE==1)  
+  dsum_k_NEHALEM,
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
+  dcopy_k_NEHALEM, ddot_k_NEHALEM,
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_DOUBLE==1)  
+  dsdot_k_NEHALEM,
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
+  drot_k_NEHALEM,
+  daxpy_k_NEHALEM,
+  dscal_k_NEHALEM, 
+  dswap_k_NEHALEM,
+  dgemv_n_NEHALEM,  dgemv_t_NEHALEM,
+#endif
+#if  (BUILD_DOUBLE==1)  
+  dger_k_NEHALEM,
   dsymv_L_NEHALEM,  dsymv_U_NEHALEM,
+#endif
 
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)  
   dgemm_kernel_NEHALEM, dgemm_beta_NEHALEM,
 #if DGEMM_DEFAULT_UNROLL_M != DGEMM_DEFAULT_UNROLL_N
   dgemm_incopy_NEHALEM, dgemm_itcopy_NEHALEM,
@@ -138,6 +254,9 @@ gotoblas_t TABLE_NAME = {
   dgemm_oncopy_NEHALEM, dgemm_otcopy_NEHALEM,
 #endif
   dgemm_oncopy_NEHALEM, dgemm_otcopy_NEHALEM,
+#endif
+
+#if  (BUILD_DOUBLE==1)  
   dtrsm_kernel_LN_NEHALEM, dtrsm_kernel_LT_NEHALEM, dtrsm_kernel_RN_NEHALEM, dtrsm_kernel_RT_NEHALEM,
 #if DGEMM_DEFAULT_UNROLL_M != DGEMM_DEFAULT_UNROLL_N
   dtrsm_iunucopy_NEHALEM, dtrsm_iunncopy_NEHALEM, dtrsm_iutucopy_NEHALEM, dtrsm_iutncopy_NEHALEM,
@@ -171,6 +290,8 @@ gotoblas_t TABLE_NAME = {
   NULL, NULL,
 #endif
 
+#endif
+
 #ifdef EXPRECISION
 
   0, 0, 0,
@@ -178,7 +299,7 @@ gotoblas_t TABLE_NAME = {
 
   qamax_k_NEHALEM,  qamin_k_NEHALEM,  qmax_k_NEHALEM,  qmin_k_NEHALEM,
   iqamax_k_NEHALEM, iqamin_k_NEHALEM, iqmax_k_NEHALEM, iqmin_k_NEHALEM,
-  qnrm2_k_NEHALEM,  qasum_k_NEHALEM,  qcopy_k_NEHALEM, qdot_k_NEHALEM,
+  qnrm2_k_NEHALEM,  qasum_k_NEHALEM,  qsum_k_NEHALEM, qcopy_k_NEHALEM, qdot_k_NEHALEM,
   qrot_k_NEHALEM,   qaxpy_k_NEHALEM,  qscal_k_NEHALEM, qswap_k_NEHALEM,
   qgemv_n_NEHALEM,  qgemv_t_NEHALEM,  qger_k_NEHALEM,
   qsymv_L_NEHALEM,  qsymv_U_NEHALEM,
@@ -225,6 +346,7 @@ gotoblas_t TABLE_NAME = {
 
 #endif
 
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
   0, 0, 0,
   CGEMM_DEFAULT_UNROLL_M, CGEMM_DEFAULT_UNROLL_N,
 #ifdef CGEMM_DEFAULT_UNROLL_MN
@@ -232,21 +354,34 @@ gotoblas_t TABLE_NAME = {
 #else
  MAX(CGEMM_DEFAULT_UNROLL_M, CGEMM_DEFAULT_UNROLL_N),
 #endif
-
   camax_k_NEHALEM, camin_k_NEHALEM, icamax_k_NEHALEM, icamin_k_NEHALEM,
-  cnrm2_k_NEHALEM, casum_k_NEHALEM, ccopy_k_NEHALEM,
-  cdotu_k_NEHALEM, cdotc_k_NEHALEM, csrot_k_NEHALEM,
-  caxpy_k_NEHALEM, caxpyc_k_NEHALEM, cscal_k_NEHALEM, cswap_k_NEHALEM,
+#endif
+#if (BUILD_COMPLEX)
+  cnrm2_k_NEHALEM, casum_k_NEHALEM, csum_k_NEHALEM,
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
+  ccopy_k_NEHALEM,  cdotu_k_NEHALEM, cdotc_k_NEHALEM,
+#endif
+#if (BUILD_COMPLEX)
+ csrot_k_NEHALEM,
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
+  caxpy_k_NEHALEM,
+  caxpyc_k_NEHALEM, 
+  cscal_k_NEHALEM, 
+  cswap_k_NEHALEM,
 
   cgemv_n_NEHALEM, cgemv_t_NEHALEM, cgemv_r_NEHALEM, cgemv_c_NEHALEM,
   cgemv_o_NEHALEM, cgemv_u_NEHALEM, cgemv_s_NEHALEM, cgemv_d_NEHALEM,
+#endif
+#if (BUILD_COMPLEX)
   cgeru_k_NEHALEM, cgerc_k_NEHALEM, cgerv_k_NEHALEM, cgerd_k_NEHALEM,
   csymv_L_NEHALEM, csymv_U_NEHALEM,
   chemv_L_NEHALEM, chemv_U_NEHALEM, chemv_M_NEHALEM, chemv_V_NEHALEM,
-
+#endif
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
   cgemm_kernel_n_NEHALEM, cgemm_kernel_l_NEHALEM, cgemm_kernel_r_NEHALEM, cgemm_kernel_b_NEHALEM,
   cgemm_beta_NEHALEM,
-
 #if CGEMM_DEFAULT_UNROLL_M != CGEMM_DEFAULT_UNROLL_N
   cgemm_incopy_NEHALEM, cgemm_itcopy_NEHALEM,
 #else
@@ -266,6 +401,8 @@ gotoblas_t TABLE_NAME = {
 #endif
   ctrsm_ounucopy_NEHALEM,  ctrsm_ounncopy_NEHALEM,  ctrsm_outucopy_NEHALEM,  ctrsm_outncopy_NEHALEM,
   ctrsm_olnucopy_NEHALEM,  ctrsm_olnncopy_NEHALEM,  ctrsm_oltucopy_NEHALEM,  ctrsm_oltncopy_NEHALEM,
+#endif
+#if (BUILD_COMPLEX)
 
   ctrmm_kernel_RN_NEHALEM,  ctrmm_kernel_RT_NEHALEM,  ctrmm_kernel_RR_NEHALEM,  ctrmm_kernel_RC_NEHALEM,
   ctrmm_kernel_LN_NEHALEM,  ctrmm_kernel_LT_NEHALEM,  ctrmm_kernel_LR_NEHALEM,  ctrmm_kernel_LC_NEHALEM,
@@ -294,6 +431,8 @@ gotoblas_t TABLE_NAME = {
   chemm_outcopy_NEHALEM,  chemm_oltcopy_NEHALEM,
 
   0, 0, 0,
+
+#if (USE_GEMM3M)
 #ifdef CGEMM3M_DEFAULT_UNROLL_M
   CGEMM3M_DEFAULT_UNROLL_M, CGEMM3M_DEFAULT_UNROLL_N, MAX(CGEMM3M_DEFAULT_UNROLL_M, CGEMM3M_DEFAULT_UNROLL_N),
 #else
@@ -324,13 +463,47 @@ gotoblas_t TABLE_NAME = {
   chemm3m_oucopyb_NEHALEM,  chemm3m_olcopyb_NEHALEM,
   chemm3m_oucopyr_NEHALEM,  chemm3m_olcopyr_NEHALEM,
   chemm3m_oucopyi_NEHALEM,  chemm3m_olcopyi_NEHALEM,
+#else
+  0, 0, 0,
 
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
+#endif
+
+#if (BUILD_COMPLEX || BUILD_COMPLEX16)
 #ifndef NO_LAPACK
-  cneg_tcopy_NEHALEM, claswp_ncopy_NEHALEM,
+  cneg_tcopy_NEHALEM,
+  
+   claswp_ncopy_NEHALEM,
 #else
   NULL, NULL,
 #endif
 
+#endif
+
+#if BUILD_COMPLEX16 == 1
   0, 0, 0,
   ZGEMM_DEFAULT_UNROLL_M, ZGEMM_DEFAULT_UNROLL_N,
 #ifdef ZGEMM_DEFAULT_UNROLL_MN
@@ -340,7 +513,7 @@ gotoblas_t TABLE_NAME = {
 #endif
 
   zamax_k_NEHALEM, zamin_k_NEHALEM, izamax_k_NEHALEM, izamin_k_NEHALEM,
-  znrm2_k_NEHALEM, zasum_k_NEHALEM, zcopy_k_NEHALEM,
+  znrm2_k_NEHALEM, zasum_k_NEHALEM, zsum_k_NEHALEM, zcopy_k_NEHALEM,
   zdotu_k_NEHALEM, zdotc_k_NEHALEM, zdrot_k_NEHALEM,
   zaxpy_k_NEHALEM, zaxpyc_k_NEHALEM, zscal_k_NEHALEM, zswap_k_NEHALEM,
 
@@ -400,6 +573,7 @@ gotoblas_t TABLE_NAME = {
   zhemm_outcopy_NEHALEM,  zhemm_oltcopy_NEHALEM,
 
   0, 0, 0,
+#if (USE_GEMM3M)
 #ifdef ZGEMM3M_DEFAULT_UNROLL_M
   ZGEMM3M_DEFAULT_UNROLL_M, ZGEMM3M_DEFAULT_UNROLL_N, MAX(ZGEMM3M_DEFAULT_UNROLL_M, ZGEMM3M_DEFAULT_UNROLL_N),
 #else
@@ -430,6 +604,33 @@ gotoblas_t TABLE_NAME = {
   zhemm3m_oucopyb_NEHALEM,  zhemm3m_olcopyb_NEHALEM,
   zhemm3m_oucopyr_NEHALEM,  zhemm3m_olcopyr_NEHALEM,
   zhemm3m_oucopyi_NEHALEM,  zhemm3m_olcopyi_NEHALEM,
+#else
+  0, 0, 0,
+
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
 
 #ifndef NO_LAPACK
   zneg_tcopy_NEHALEM, zlaswp_ncopy_NEHALEM,
@@ -437,13 +638,15 @@ gotoblas_t TABLE_NAME = {
   NULL, NULL,
 #endif
 
+#endif
+
 #ifdef EXPRECISION
 
   0, 0, 0,
   XGEMM_DEFAULT_UNROLL_M, XGEMM_DEFAULT_UNROLL_N, MAX(XGEMM_DEFAULT_UNROLL_M, XGEMM_DEFAULT_UNROLL_N),
 
   xamax_k_NEHALEM, xamin_k_NEHALEM, ixamax_k_NEHALEM, ixamin_k_NEHALEM,
-  xnrm2_k_NEHALEM, xasum_k_NEHALEM, xcopy_k_NEHALEM,
+  xnrm2_k_NEHALEM, xasum_k_NEHALEM, xsum_k_NEHALEM, xcopy_k_NEHALEM,
   xdotu_k_NEHALEM, xdotc_k_NEHALEM, xqrot_k_NEHALEM,
   xaxpy_k_NEHALEM, xaxpyc_k_NEHALEM, xscal_k_NEHALEM, xswap_k_NEHALEM,
 
@@ -503,6 +706,7 @@ gotoblas_t TABLE_NAME = {
   xhemm_outcopy_NEHALEM,  xhemm_oltcopy_NEHALEM,
 
   0, 0, 0,
+#if (USE_GEMM3M)
   QGEMM_DEFAULT_UNROLL_M, QGEMM_DEFAULT_UNROLL_N, MAX(QGEMM_DEFAULT_UNROLL_M, QGEMM_DEFAULT_UNROLL_N),
 
   xgemm3m_kernel_NEHALEM,
@@ -528,6 +732,33 @@ gotoblas_t TABLE_NAME = {
   xhemm3m_oucopyb_NEHALEM,  xhemm3m_olcopyb_NEHALEM,
   xhemm3m_oucopyr_NEHALEM,  xhemm3m_olcopyr_NEHALEM,
   xhemm3m_oucopyi_NEHALEM,  xhemm3m_olcopyi_NEHALEM,
+#else
+  0, 0, 0,
+
+  NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+
+  NULL, NULL,
+  NULL, NULL,
+  NULL, NULL,
+#endif
 
 #ifndef NO_LAPACK
   xneg_tcopy_NEHALEM, xlaswp_ncopy_NEHALEM,
@@ -540,27 +771,298 @@ gotoblas_t TABLE_NAME = {
   init_parameter,
 
   SNUMOPT, DNUMOPT, QNUMOPT,
+#if BUILD_SINGLE == 1
+  saxpby_k_NEHALEM,
+#endif
+#if BUILD_DOUBLE  == 1
+  daxpby_k_NEHALEM,
+#endif
+#if BUILD_COMPLEX == 1
+  caxpby_k_NEHALEM,
+#endif
+#if BUILD_COMPLEX16== 1
+  zaxpby_k_NEHALEM,
+#endif
 
-  saxpby_k_NEHALEM, daxpby_k_NEHALEM, caxpby_k_NEHALEM, zaxpby_k_NEHALEM,
-
+#if BUILD_SINGLE == 1
   somatcopy_k_cn_NEHALEM, somatcopy_k_ct_NEHALEM, somatcopy_k_rn_NEHALEM, somatcopy_k_rt_NEHALEM,
+#endif
+#if BUILD_DOUBLE== 1
   domatcopy_k_cn_NEHALEM, domatcopy_k_ct_NEHALEM, domatcopy_k_rn_NEHALEM, domatcopy_k_rt_NEHALEM,
+#endif
+#if BUILD_COMPLEX == 1
   comatcopy_k_cn_NEHALEM, comatcopy_k_ct_NEHALEM, comatcopy_k_rn_NEHALEM, comatcopy_k_rt_NEHALEM,
   comatcopy_k_cnc_NEHALEM, comatcopy_k_ctc_NEHALEM, comatcopy_k_rnc_NEHALEM, comatcopy_k_rtc_NEHALEM,
+#endif
+#if BUILD_COMPLEX16 == 1
   zomatcopy_k_cn_NEHALEM, zomatcopy_k_ct_NEHALEM, zomatcopy_k_rn_NEHALEM, zomatcopy_k_rt_NEHALEM,
   zomatcopy_k_cnc_NEHALEM, zomatcopy_k_ctc_NEHALEM, zomatcopy_k_rnc_NEHALEM, zomatcopy_k_rtc_NEHALEM,
+#endif
 
+#if BUILD_SINGLE == 1
   simatcopy_k_cn_NEHALEM, simatcopy_k_ct_NEHALEM, simatcopy_k_rn_NEHALEM, simatcopy_k_rt_NEHALEM,
+#endif
+#if BUILD_DOUBLE== 1
   dimatcopy_k_cn_NEHALEM, dimatcopy_k_ct_NEHALEM, dimatcopy_k_rn_NEHALEM, dimatcopy_k_rt_NEHALEM,
+#endif
+#if BUILD_COMPLEX== 1
   cimatcopy_k_cn_NEHALEM, cimatcopy_k_ct_NEHALEM, cimatcopy_k_rn_NEHALEM, cimatcopy_k_rt_NEHALEM,
   cimatcopy_k_cnc_NEHALEM, cimatcopy_k_ctc_NEHALEM, cimatcopy_k_rnc_NEHALEM, cimatcopy_k_rtc_NEHALEM,
+#endif
+#if BUILD_COMPLEX16==1
   zimatcopy_k_cn_NEHALEM, zimatcopy_k_ct_NEHALEM, zimatcopy_k_rn_NEHALEM, zimatcopy_k_rt_NEHALEM,
   zimatcopy_k_cnc_NEHALEM, zimatcopy_k_ctc_NEHALEM, zimatcopy_k_rnc_NEHALEM, zimatcopy_k_rtc_NEHALEM,
+#endif
 
-  sgeadd_k_NEHALEM, dgeadd_k_NEHALEM, cgeadd_k_NEHALEM, zgeadd_k_NEHALEM
-
+#if BUILD_SINGLE == 1
+  sgeadd_k_NEHALEM,
+#endif
+#if BUILD_DOUBLE==1
+  dgeadd_k_NEHALEM,
+#endif
+#if BUILD_COMPLEX==1
+  cgeadd_k_NEHALEM,
+#endif
+#if BUILD_COMPLEX16==1
+  zgeadd_k_NEHALEM
+#endif
 };
 
+#if (ARCH_ARM64)
+static void init_parameter(void) {
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+#if (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE == 1
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+#endif
+#if BUILD_DOUBLE== 1
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX== 1
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+#endif
+
+#if (BUILD_BFLOAT16)
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+#endif
+#if BUILD_DOUBLE==1 
+  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+  TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
+  TABLE_NAME.xgemm_q = XGEMM_DEFAULT_Q;
+  TABLE_NAME.qgemm_r = QGEMM_DEFAULT_R;
+  TABLE_NAME.xgemm_r = XGEMM_DEFAULT_R;
+#endif
+
+#if (USE_GEMM3M)
+#ifdef CGEMM3M_DEFAULT_P
+  TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_P
+  TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_Q
+  TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.cgemm3m_q = TABLE_NAME.sgemm_q;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_Q
+  TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.zgemm3m_q = TABLE_NAME.dgemm_q;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_R
+  TABLE_NAME.cgemm3m_r = CGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.cgemm3m_r = TABLE_NAME.sgemm_r;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_R
+  TABLE_NAME.zgemm3m_r = ZGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.zgemm3m_r = TABLE_NAME.dgemm_r;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
+  TABLE_NAME.xgemm3m_q = TABLE_NAME.qgemm_q;
+  TABLE_NAME.xgemm3m_r = TABLE_NAME.qgemm_r;
+#endif
+#endif
+
+}
+#else // (ARCH_ARM64)
+#if defined(ARCH_MIPS64)
+static void init_parameter(void) {
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+  TABLE_NAME.dgemm_r = 640;
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+  TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
+  TABLE_NAME.xgemm_q = XGEMM_DEFAULT_Q;
+  TABLE_NAME.qgemm_r = QGEMM_DEFAULT_R;
+  TABLE_NAME.xgemm_r = XGEMM_DEFAULT_R;
+#endif
+
+#if defined(USE_GEMM3M)
+#ifdef CGEMM3M_DEFAULT_P
+  TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_P
+  TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
+#else
+  TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_Q
+  TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.cgemm3m_q = TABLE_NAME.sgemm_q;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_Q
+  TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
+#else
+  TABLE_NAME.zgemm3m_q = TABLE_NAME.dgemm_q;
+#endif
+
+#ifdef CGEMM3M_DEFAULT_R
+  TABLE_NAME.cgemm3m_r = CGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.cgemm3m_r = TABLE_NAME.sgemm_r;
+#endif
+
+#ifdef ZGEMM3M_DEFAULT_R
+  TABLE_NAME.zgemm3m_r = ZGEMM3M_DEFAULT_R;
+#else
+  TABLE_NAME.zgemm3m_r = TABLE_NAME.dgemm_r;
+#endif
+
+#ifdef EXPRECISION
+  TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
+  TABLE_NAME.xgemm3m_q = TABLE_NAME.qgemm_q;
+  TABLE_NAME.xgemm3m_r = TABLE_NAME.qgemm_r;
+#endif
+#endif
+}
+#else // (ARCH_MIPS64)
+#if (ARCH_POWER)
+static void init_parameter(void) {
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+}
+#else //POWER
+
+#if (ARCH_ZARCH)
+static void init_parameter(void) {
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+	TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+	TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+	TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+	TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+	TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+	TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+	TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+	TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+
+#ifdef BUILD_BFLOAT16
+	TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+	TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+	TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+	TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+	TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+}
+#else //ZARCH
+
 #ifdef ARCH_X86
 static int get_l2_size_old(void){
   int i, eax, ebx, ecx, edx, cpuid_level;
@@ -647,7 +1149,9 @@ static int get_l2_size_old(void){
       return 6144;
     }
   }
-  return 0;
+//  return 0;
+fprintf (stderr,"OpenBLAS WARNING - could not determine the L2 cache size on this system, assuming 256k\n");
+return 256;
 }
 #endif
 
@@ -660,6 +1164,10 @@ static __inline__ int get_l2_size(void){
   l2 = BITMASK(ecx, 16, 0xffff);
 
 #ifndef ARCH_X86
+  if (l2 <= 0) {
+     fprintf (stderr,"OpenBLAS WARNING - could not determine the L2 cache size on this system, assuming 256k\n");
+     return 256;
+  }
   return l2;
 
 #else
@@ -684,22 +1192,42 @@ static void init_parameter(void) {
 
   int l2 = get_l2_size();
 
+  (void) l2; /* dirty trick to suppress unused variable warning for targets */
+             /* where the GEMM unrolling parameters do not depend on l2 */
+  
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+#endif
+#if  (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)
   TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX == 1
   TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+#endif
 
+#if BUILD_COMPLEX == 1
 #ifdef CGEMM3M_DEFAULT_Q
   TABLE_NAME.cgemm3m_q = CGEMM3M_DEFAULT_Q;
 #else
   TABLE_NAME.cgemm3m_q = SGEMM_DEFAULT_Q;
 #endif
+#endif
 
+#if BUILD_COMPLEX16 == 1
 #ifdef ZGEMM3M_DEFAULT_Q
   TABLE_NAME.zgemm3m_q = ZGEMM3M_DEFAULT_Q;
 #else
   TABLE_NAME.zgemm3m_q = DGEMM_DEFAULT_Q;
 #endif
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_q = QGEMM_DEFAULT_Q;
@@ -713,10 +1241,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Katmai, Coppermine, Banias, Athlon\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  64 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  32 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1 
   TABLE_NAME.cgemm_p =  32 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  16 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  16 * (l2 >> 7);
   TABLE_NAME.xgemm_p =   8 * (l2 >> 7);
@@ -729,10 +1265,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Northwood\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  96 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  48 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1 
   TABLE_NAME.cgemm_p =  48 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  24 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  24 * (l2 >> 7);
   TABLE_NAME.xgemm_p =  12 * (l2 >> 7);
@@ -745,10 +1289,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Atom\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = 256;
+#endif
+#if BUILD_DOUBLE ==1
   TABLE_NAME.dgemm_p = 128;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p = 128;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  64;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  64;
   TABLE_NAME.xgemm_p =  32;
@@ -761,10 +1313,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Prescott\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  56 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE ==1 
   TABLE_NAME.dgemm_p =  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16 == 1
   TABLE_NAME.zgemm_p =  14 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  14 * (l2 >> 7);
   TABLE_NAME.xgemm_p =   7 * (l2 >> 7);
@@ -777,10 +1337,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Core2\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  92 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE==1
   TABLE_NAME.dgemm_p =  46 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  46 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  23 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  92 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  46 * (l2 >> 9) + 4;
@@ -793,10 +1361,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Penryn\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE == 1
   TABLE_NAME.dgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  21 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  21 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  42 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  21 * (l2 >> 9) + 4;
@@ -809,10 +1385,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Dunnington\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_DOUBLE ==1
   TABLE_NAME.dgemm_p =  42 * (l2 >> 9) + 8;
+#endif
+#if BUILD_COMPLEX==1
   TABLE_NAME.cgemm_p =  21 * (l2 >> 9) + 4;
+#endif
+#if BUILD_COMPLEX16==1
   TABLE_NAME.zgemm_p =  21 * (l2 >> 9) + 4;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  42 * (l2 >> 9) + 8;
   TABLE_NAME.xgemm_p =  21 * (l2 >> 9) + 4;
@@ -826,10 +1410,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Nehalem\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -842,10 +1434,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Sandybridge\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -858,10 +1458,42 @@ static void init_parameter(void) {
   fprintf(stderr, "Haswell\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if (BUILD_DOUBLE==1) || (BUILD_COMPLEX16)
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+#if defined(SKYLAKEX) || defined(COOPERLAKE)
+
+#ifdef DEBUG
+  fprintf(stderr, "SkylakeX\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -875,10 +1507,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Opteron\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = 224 +  56 * (l2 >> 7);
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = 112 +  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = 112 +  28 * (l2 >> 7);
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p =  56 +  14 * (l2 >> 7);
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p =  56 +  14 * (l2 >> 7);
   TABLE_NAME.xgemm_p =  28 +   7 * (l2 >> 7);
@@ -891,10 +1531,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Barcelona\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -907,10 +1555,18 @@ static void init_parameter(void) {
   fprintf(stderr, "Bobcate\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -923,26 +1579,67 @@ static void init_parameter(void) {
   fprintf(stderr, "Bulldozer\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
 #endif
 #endif
 
+#ifdef EXCAVATOR
+
+#ifdef DEBUG
+  fprintf(stderr, "Excavator\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+
 #ifdef PILEDRIVER
 
 #ifdef DEBUG
   fprintf(stderr, "Piledriver\n");
 #endif
 
+#if (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -955,10 +1652,42 @@ static void init_parameter(void) {
   fprintf(stderr, "Steamroller\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
+#ifdef EXPRECISION
+  TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
+  TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
+#endif
+#endif
+
+#ifdef ZEN
+
+#ifdef DEBUG
+  fprintf(stderr, "Zen\n");
+#endif
+
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if BUILD_DOUBLE
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if BUILD_COMPLEX16
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+#endif
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_p = QGEMM_DEFAULT_P;
   TABLE_NAME.xgemm_p = XGEMM_DEFAULT_P;
@@ -972,11 +1701,18 @@ static void init_parameter(void) {
   fprintf(stderr, "NANO\n");
 #endif
 
+#if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+#endif
+#if  (BUILD_DOUBLE==1)
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+#endif
+#if (BUILD_COMPLEX==1)
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+#endif
+#if (BUILD_COMPLEX16==1)
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
-
+#endif
 
 
 #ifdef EXPRECISION
@@ -986,61 +1722,79 @@ static void init_parameter(void) {
 
 #endif
 
-
+#if BUILD_COMPLEX==1
 #ifdef CGEMM3M_DEFAULT_P
   TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
 #else
   TABLE_NAME.cgemm3m_p = TABLE_NAME.sgemm_p;
 #endif
+#endif
 
+#if BUILD_COMPLEX16==1
 #ifdef ZGEMM3M_DEFAULT_P
   TABLE_NAME.zgemm3m_p = ZGEMM3M_DEFAULT_P;
 #else
   TABLE_NAME.zgemm3m_p = TABLE_NAME.dgemm_p;
 #endif
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.xgemm3m_p = TABLE_NAME.qgemm_p;
 #endif
 
 
-
-  TABLE_NAME.sgemm_p = (TABLE_NAME.sgemm_p + SGEMM_DEFAULT_UNROLL_M - 1) & ~(SGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.dgemm_p = (TABLE_NAME.dgemm_p + DGEMM_DEFAULT_UNROLL_M - 1) & ~(DGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.cgemm_p = (TABLE_NAME.cgemm_p + CGEMM_DEFAULT_UNROLL_M - 1) & ~(CGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.zgemm_p = (TABLE_NAME.zgemm_p + ZGEMM_DEFAULT_UNROLL_M - 1) & ~(ZGEMM_DEFAULT_UNROLL_M - 1);
-
-#ifdef CGEMM3M_DEFAULT_UNROLL_M
-  TABLE_NAME.cgemm3m_p = (TABLE_NAME.cgemm3m_p + CGEMM3M_DEFAULT_UNROLL_M - 1) & ~(CGEMM3M_DEFAULT_UNROLL_M - 1);
-#else
-  TABLE_NAME.cgemm3m_p = (TABLE_NAME.cgemm3m_p + SGEMM_DEFAULT_UNROLL_M - 1) & ~(SGEMM_DEFAULT_UNROLL_M - 1);
+#if BUILD_SINGLE == 1
+  TABLE_NAME.sgemm_p = ((TABLE_NAME.sgemm_p + SGEMM_DEFAULT_UNROLL_M - 1)/SGEMM_DEFAULT_UNROLL_M) * SGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_DOUBLE== 1
+  TABLE_NAME.dgemm_p = ((TABLE_NAME.dgemm_p + DGEMM_DEFAULT_UNROLL_M - 1)/DGEMM_DEFAULT_UNROLL_M) * DGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_COMPLEX==1
+  TABLE_NAME.cgemm_p = ((TABLE_NAME.cgemm_p + CGEMM_DEFAULT_UNROLL_M - 1)/CGEMM_DEFAULT_UNROLL_M) * CGEMM_DEFAULT_UNROLL_M;
+#endif
+#if BUILD_COMPLEX16==1
+  TABLE_NAME.zgemm_p = ((TABLE_NAME.zgemm_p + ZGEMM_DEFAULT_UNROLL_M - 1)/ZGEMM_DEFAULT_UNROLL_M) * ZGEMM_DEFAULT_UNROLL_M;
 #endif
 
-#ifdef ZGEMM3M_DEFAULT_UNROLL_M
-  TABLE_NAME.zgemm3m_p = (TABLE_NAME.zgemm3m_p + ZGEMM3M_DEFAULT_UNROLL_M - 1) & ~(ZGEMM3M_DEFAULT_UNROLL_M - 1);
+#if BUILD_COMPLEX==1
+#ifdef CGEMM3M_DEFAULT_UNROLL_M
+  TABLE_NAME.cgemm3m_p = ((TABLE_NAME.cgemm3m_p + CGEMM3M_DEFAULT_UNROLL_M - 1)/CGEMM3M_DEFAULT_UNROLL_M) * CGEMM3M_DEFAULT_UNROLL_M;
 #else
-  TABLE_NAME.zgemm3m_p = (TABLE_NAME.zgemm3m_p + DGEMM_DEFAULT_UNROLL_M - 1) & ~(DGEMM_DEFAULT_UNROLL_M - 1);
+  TABLE_NAME.cgemm3m_p = ((TABLE_NAME.cgemm3m_p + SGEMM_DEFAULT_UNROLL_M - 1)/SGEMM_DEFAULT_UNROLL_M) * SGEMM_DEFAULT_UNROLL_M;
+#endif
+#endif
+
+#if BUILD_COMPLEX16==1
+#ifdef ZGEMM3M_DEFAULT_UNROLL_M
+  TABLE_NAME.zgemm3m_p = ((TABLE_NAME.zgemm3m_p + ZGEMM3M_DEFAULT_UNROLL_M - 1)/ZGEMM3M_DEFAULT_UNROLL_M) * ZGEMM3M_DEFAULT_UNROLL_M;
+#else
+  TABLE_NAME.zgemm3m_p = ((TABLE_NAME.zgemm3m_p + DGEMM_DEFAULT_UNROLL_M - 1)/DGEMM_DEFAULT_UNROLL_M) * DGEMM_DEFAULT_UNROLL_M;
+#endif
 #endif
 
 #ifdef QUAD_PRECISION
-  TABLE_NAME.qgemm_p = (TABLE_NAME.qgemm_p + QGEMM_DEFAULT_UNROLL_M - 1) & ~(QGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.xgemm_p = (TABLE_NAME.xgemm_p + XGEMM_DEFAULT_UNROLL_M - 1) & ~(XGEMM_DEFAULT_UNROLL_M - 1);
-  TABLE_NAME.xgemm3m_p = (TABLE_NAME.xgemm3m_p + QGEMM_DEFAULT_UNROLL_M - 1) & ~(QGEMM_DEFAULT_UNROLL_M - 1);
+  TABLE_NAME.qgemm_p = ((TABLE_NAME.qgemm_p + QGEMM_DEFAULT_UNROLL_M - 1)/QGEMM_DEFAULT_UNROLL_M) * QGEMM_DEFAULT_UNROLL_M;
+  TABLE_NAME.xgemm_p = ((TABLE_NAME.xgemm_p + XGEMM_DEFAULT_UNROLL_M - 1)/XGEMM_DEFAULT_UNROLL_M) * XGEMM_DEFAULT_UNROLL_M;
+  TABLE_NAME.xgemm3m_p = ((TABLE_NAME.xgemm3m_p + QGEMM_DEFAULT_UNROLL_M - 1)/QGEMM_DEFAULT_UNROLL_M) * QGEMM_DEFAULT_UNROLL_M;
 #endif
 
 #ifdef DEBUG
   fprintf(stderr, "L2 = %8d DGEMM_P  .. %d\n", l2, TABLE_NAME.dgemm_p);
 #endif
 
+#if BUILD_SINGLE==1
   TABLE_NAME.sgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.sgemm_p * TABLE_NAME.sgemm_q *  4 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.sgemm_q *  4) - 15) & ~15);
+#endif
 
+#if BUILD_DOUBLE==1
   TABLE_NAME.dgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.dgemm_p * TABLE_NAME.dgemm_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.dgemm_q *  8) - 15) & ~15);
+#endif
 
 #ifdef EXPRECISION
   TABLE_NAME.qgemm_r = (((BUFFER_SIZE -
@@ -1049,26 +1803,33 @@ static void init_parameter(void) {
 			       ) / (TABLE_NAME.qgemm_q * 16) - 15) & ~15);
 #endif
 
+#if BUILD_COMPLEX ==1 
   TABLE_NAME.cgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.cgemm_p * TABLE_NAME.cgemm_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.cgemm_q *  8) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX16 ==1
   TABLE_NAME.zgemm_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.zgemm_p * TABLE_NAME.zgemm_q * 16 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.zgemm_q * 16) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX == 1
   TABLE_NAME.cgemm3m_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.cgemm3m_p * TABLE_NAME.cgemm3m_q *  8 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.cgemm3m_q *  8) - 15) & ~15);
+#endif
 
+#if BUILD_COMPLEX16 == 1
   TABLE_NAME.zgemm3m_r = (((BUFFER_SIZE -
 			       ((TABLE_NAME.zgemm3m_p * TABLE_NAME.zgemm3m_q * 16 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.zgemm3m_q * 16) - 15) & ~15);
-
+#endif
 
 
 
@@ -1088,3 +1849,7 @@ static void init_parameter(void) {
 
 
 }
+#endif //POWER
+#endif //ZARCH
+#endif //(ARCH_MIPS64)
+#endif //(ARCH_ARM64)
